<?php
defined('ABSPATH') or die();
function aiomatic_get_blog_timezone() {

    $tzstring = get_option( 'timezone_string' );
    $offset   = get_option( 'gmt_offset' );

    if( empty( $tzstring ) && 0 != $offset && floor( $offset ) == $offset ){
        $offset_st = $offset > 0 ? "-$offset" : '+'.absint( $offset );
        $tzstring  = 'Etc/GMT'.$offset_st;
    }
    if( empty( $tzstring ) ){
        $tzstring = 'UTC';
    }
    $timezone = new DateTimeZone( $tzstring );
    return $timezone; 
}
function aiomatic_add_processed_keyword($keyword) 
{
    $processed_keywords = get_option('aiomatic_processed_keywords', array());
    if (!in_array($keyword, $processed_keywords)) 
    {
        $processed_keywords[] = $keyword;
        update_option('aiomatic_processed_keywords', $processed_keywords, false);
    }
}
function aiomatic_remove_processed_keyword($keyword) 
{
    $processed_keywords = get_option('aiomatic_processed_keywords', array());
    if (($key = array_search($keyword, $processed_keywords)) !== false) 
    {
        unset($processed_keywords[$key]);
        update_option('aiomatic_processed_keywords', $processed_keywords, false);
    }
}
function aiomatic_remove_processed_keywords($keywords) 
{
    $processed_keywords = get_option('aiomatic_processed_keywords', array());
    $updated = false;
    foreach($keywords as $keyword)
    {
        if (($key = array_search($keyword, $processed_keywords)) !== false) 
        {
            unset($processed_keywords[$key]);
            $updated = true;
        }
    }
    if($updated == true)
    {
        update_option('aiomatic_processed_keywords', $processed_keywords, false);
    }
}
function aiomatic_is_keyword_processed($keyword) 
{
    $processed_keywords = get_option('aiomatic_processed_keywords', array());
    return in_array($keyword, $processed_keywords);
}
function aiomatic_array_search_recursive($needle, $haystack) 
{
    foreach ($haystack as $value) {
        if (is_array($value) && aiomatic_array_search_recursive($needle, $value)) return true;
        else if ($value == $needle) return true;
    }
    return false;
}
function aiomatic_generatePostTitleFromUrl($url) 
{
    $path = parse_url($url, PHP_URL_PATH);
    $file = basename($path);
    $fileWithoutExtension = pathinfo($file, PATHINFO_FILENAME);
    $title = str_replace(['-', '_'], ' ', $fileWithoutExtension);
    $formattedTitle = ucwords($title);
    return $formattedTitle;
}
function aiomatic_simpleEncryptWithKey($text, $key) 
{
    $result = '';
    $keyLength = strlen($key);
    for ($i = 0, $len = strlen($text); $i < $len; $i++) 
    {
        $shift = ord($key[$i % $keyLength]);
        $char = chr((ord($text[$i]) + $shift) % 256);
        $result .= $char;
    }
    return base64_encode($result);
}
function aiomatic_simpleEncrypt($text, $shift) 
{
    $result = '';
    for ($i = 0; $i < strlen($text); $i++) {
        $char = $text[$i];
        $char = chr(ord($char) + $shift);
        $result .= $char;
    }
    return base64_encode($result);
}
function aiomatic_truncate_float( $number, $precision = 4 ) 
{
    $factor = pow( 10, $precision );
    return floor( $number * $factor ) / $factor;
}
function aiomatic_parse_pre_code_entities($htmlContent)
{
    $dom = new DOMDocument();
    $convenc = htmlentities($htmlContent, ENT_QUOTES | ENT_HTML5, 'UTF-8');
    if($convenc === false)
    {
        return $htmlContent;
    }
    $internalErrors = libxml_use_internal_errors(true);
    $dom->loadHTML('<?xml encoding="utf-8" ?>' . $convenc, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    libxml_use_internal_errors($internalErrors);
    $xpath = new DOMXPath($dom);
    $preCodeElements = $xpath->query('//pre|//code');
    if($preCodeElements === false || $preCodeElements->length == 0)
    {
        return $htmlContent;
    }
    foreach ($preCodeElements as $element) 
    {
        $fragment = $dom->createDocumentFragment();
        $fragment->appendXML(html_entity_decode($element->textContent));
        $element->parentNode->replaceChild($fragment, $element);
    }
    $cleanedHtml = $dom->saveHTML();
    return $cleanedHtml;
}
function aiomatic_makeAbsoluteUrl($originalUrl, $relativeUrl) 
{
    if (parse_url($relativeUrl, PHP_URL_SCHEME) != '') {
        return $relativeUrl;
    }
    $parts = parse_url($originalUrl);
    if ($relativeUrl[0] == '/') {
        return $parts['scheme'] . '://' . $parts['host'] . $relativeUrl;
    }
    $basePath = isset($parts['path']) ? $parts['path'] : '/';
    if ($basePath[strlen($basePath) - 1] != '/') {
        $basePath = dirname($basePath) . '/';
    }
    return $parts['scheme'] . '://' . $parts['host'] . $basePath . $relativeUrl;
}
function aiomatic_fix_relative_links($htmlContent, $originalUrl)
{
    if((stristr($htmlContent,'src=') === false && stristr($htmlContent,'href=') === false) || strip_tags($htmlContent) == $htmlContent)
    {
        return $htmlContent;
    }
    $doc = new DOMDocument();
    $internalErrors = libxml_use_internal_errors(true);
    $doc->loadHTML('<?xml encoding="utf-8" ?>' . $htmlContent, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    libxml_use_internal_errors($internalErrors);
    $xpath = new DOMXPath($doc);
    $imgNodes = $xpath->query('//img[@src]');
    $aNodes = $xpath->query('//a[@href]');
    if($imgNodes !== false)
    {
        foreach ($imgNodes as $node) 
        {
            $src = $node->getAttribute('src');
            $absoluteUrl = aiomatic_makeAbsoluteUrl($originalUrl, $src);
            $node->setAttribute('src', $absoluteUrl);
        }
    }
    if($aNodes !== false)
    {
        foreach ($aNodes as $node) 
        {
            $href = $node->getAttribute('href');
            $absoluteUrl = aiomatic_makeAbsoluteUrl($originalUrl, $href);
            $node->setAttribute('href', $absoluteUrl);
        }
    }
    return $doc->saveHTML();
}
function aiomatic_rrmdir($dir) {
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
        include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
        wp_filesystem($creds);
    }
    if ($wp_filesystem->is_dir($dir)) {
    $objects = scandir($dir);
    foreach ($objects as $object) {
      if ($object != "." && $object != "..") {
        if (filetype($dir."/".$object) == "dir") 
            aiomatic_rrmdir($dir."/".$object); 
        else $wp_filesystem->delete($dir."/".$object);
      }
    }
    reset($objects);
    $wp_filesystem->rmdir($dir);
  }
}
function aiomatic_extract_remote_xlsx($inputFile, $xlsx_sheet)
{
    if(strstr(trim($inputFile), ' ') !== false)
    {
        $inputFile = str_replace(' ', '%20', trim($inputFile));
    }
    $full_row = array();
    try
    {
        global $wp_filesystem;
        if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
            include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
            wp_filesystem($creds);
        }
        $dir = dirname(__FILE__) . "/tmp";
        if(!$wp_filesystem->is_dir( $dir )) {
            $wp_filesystem->mkdir($dir);
        }
        $newfile = $dir . '/tmpxlsx.xlsx';

        if ( $wp_filesystem->copy($inputFile, $newfile, true) ) {
            $inputFile = $newfile;
        }else{
            aiomatic_log_to_file('Failed to copy remote file locally: ' . $inputFile);
            aiomatic_rrmdir($dir);
            return false;
        }
        if(!class_exists('ZipArchive'))
        {
            aiomatic_log_to_file('ZipArchive class not found, please activate it on your server');
            aiomatic_rrmdir($dir);
            return false;
        }
        $zip = new ZipArchive();
        if($zip->open($inputFile) !== TRUE)
        {
            aiomatic_log_to_file('Failed to open archive: ' . $inputFile);
            aiomatic_rrmdir($dir);
            return false;
        }
        if($zip->extractTo($dir) !== TRUE)
        {
            aiomatic_log_to_file('Failed to extractTo archive to: ' . $dir);
            $zip->close();
            aiomatic_rrmdir($dir);
            return false;
        }
        
        $strings = simplexml_load_file($dir . '/xl/sharedStrings.xml');
        if($strings === false)
        {
            aiomatic_log_to_file('Unexpected error while simplexml_load_file sharedStrings.');
            $zip->close();
            aiomatic_rrmdir($dir);
            return false;
        }
        if($xlsx_sheet === '' || !is_numeric($xlsx_sheet))
        {
            $xlsx_sheet = '1';
        }
        $sheet   = simplexml_load_file($dir . '/xl/worksheets/sheet' . $xlsx_sheet . '.xml');
        if($sheet === false)
        {
            aiomatic_log_to_file('Unexpected error while simplexml_load_file sheet1.');
            $zip->close();
            aiomatic_rrmdir($dir);
            return false;
        }
        $xlrows = $sheet->sheetData->row;
        $headers = array();
        foreach ($xlrows as $xlrow) {
            $arr = array();
            foreach ($xlrow->c as $cell) {
                $v = (string) $cell->v;
                if (isset($cell['t']) && $cell['t'] == 's') {
                    $s  = array();
                    $si = $strings->si[(int) $v];
                    $si->registerXPathNamespace('n', 'http://schemas.openxmlformats.org/spreadsheetml/2006/main');
                    foreach($si->xpath('.//n:t') as $t) {
                        $s[] = (string) $t;
                    }
                    $v = implode('',$s);
                }
                $v = trim(preg_replace('/\s+/', ' ', $v));
                $arr[] = $v;
            }
            while ("" === end($arr))
            {
                array_pop($arr);
            }
            $full_row[] = $arr;
        }
        $zip->close();
        aiomatic_rrmdir($dir);
    }
    catch(Exception $e)
    {
        aiomatic_log_to_file('Unexpected error while parsing xlsx: ' . $e->getMessage());
        return false;
    }
    return $full_row;
}
function aiomatic_fix_links($str, $url)
{
    require_once (dirname(__FILE__) . '/res/Net_URL2.php');
    require_once (dirname(__FILE__) . "/res/simple_html_dom.php");
    $replaced_links = array();
    $extractok = false;
    $html_dom_original_html = aiomatic_str_get_html($str);
    if($html_dom_original_html !== false && method_exists($html_dom_original_html, 'find'))
    {
        foreach($html_dom_original_html->find('a') as $a) 
        {
            $extractok = true;
            if($a->href) 
            {
                if($a->href == '#') 
                {
                    continue;
                }
                if(preg_match("/^(about|javascript|magnet|mailto|sms|tel|geo):/i", $a->href))
                {
                    continue;
                }
                if(!in_array($a->href, $replaced_links))
                {
                    $replaced_links[] = $a->href;
                    try {
                        $relUrl = new Net_URL2($a->href);
                        if ($relUrl->isAbsolute()) {
                            continue;
                        }
                        $baseUrl = new Net_URL2($url);
                        $absUrl = $baseUrl->resolve($relUrl);
                        $full_url = $absUrl->getURL();
                        if($full_url != $a->href)
                        {
                            if($a->href == '/')
                            {
                                $str = str_replace('href="' . $a->href .'"', 'href="' . $full_url . '"', $str);
                                $str = str_replace("href='" . $a->href ."'", 'href="' . $full_url . '"', $str);
                            }
                            else
                            {
                                $str = str_replace($a->href, $full_url, $str);
                            }
                        }
                    } catch (Exception $e) {
                        aiomatic_log_to_file('Unable to resolve relative link "' . $a->href . '" against base "' . $url . '": ' . $e->getMessage());
                        continue;
                    }
                }
            }
        }
        foreach($html_dom_original_html->find('img') as $img) 
        {
            $extractok = true;
            if($img->src) 
            {
                if(aiomatic_starts_with($img->src, 'data:image'))
                {
                    continue;
                }
                if(!in_array($img->src, $replaced_links))
                {
                    $replaced_links[] = $img->src;
                    try {
                        $relUrl = new Net_URL2($img->src);
                        if ($relUrl->isAbsolute()) {
                            continue;
                        }
                        $baseUrl = new Net_URL2($url);
                        $absUrl = $baseUrl->resolve($relUrl);
                        $full_url = $absUrl->getURL();
                        if($full_url != $img->src)
                        {
                            $str = str_replace($img->src, $full_url, $str);
                        }
                    } catch (Exception $e) {
                        aiomatic_log_to_file('Unable to resolve relative image link "' . $img->src . '" against base "' . $url . '": ' . $e->getMessage());
                        continue;
                    }
                }
            }
        }
    }
    if($extractok == false)
    {
        $htmlDom = new DOMDocument;
        $internalErrors = libxml_use_internal_errors(true);
        $htmlDom->loadHTML('<?xml encoding="utf-8" ?>' . $str);
        libxml_use_internal_errors($internalErrors);
        $links = $htmlDom->getElementsByTagName('a');
        foreach($links as $link)
        {
            $linkHref = $link->getAttribute('href');
            if(strlen(trim($linkHref)) == 0){
                continue;
            }
            if($linkHref[0] == '#'){
                continue;
            }
            if(preg_match("/^(about|javascript|magnet|mailto|sms|tel|geo):/i", $linkHref))
            {
                continue;
            }
            if(!in_array($linkHref, $replaced_links))
            {
                $replaced_links[] = $linkHref;
                try {
                    $relUrl = new Net_URL2($linkHref);
                    if ($relUrl->isAbsolute()) {
                        continue;
                    }
                    $baseUrl = new Net_URL2($url);
                    $absUrl = $baseUrl->resolve($relUrl);
                    $full_url = $absUrl->getURL();
                    if($full_url != $linkHref)
                    {
                        $str = str_replace($linkHref, $full_url, $str);
                    }
                } catch (Exception $e) {
                    aiomatic_log_to_file('Unable to resolve (2) relative link "' . $linkHref . '" against base "' . $url . '": ' . $e->getMessage());
                    continue;
                }
            }
        }
        $links = $htmlDom->getElementsByTagName('img');
        foreach($links as $link)
        {
            $linkHref = $link->getAttribute('src');
            if(strlen(trim($linkHref)) == 0){
                continue;
            }
            if(aiomatic_starts_with($linkHref, 'data:image'))
            {
                continue;
            }
            if(!in_array($linkHref, $replaced_links))
            {
                $replaced_links[] = $linkHref;
                try {
                    $relUrl = new Net_URL2($linkHref);
                    if ($relUrl->isAbsolute()) {
                        continue;
                    }
                    $baseUrl = new Net_URL2($url);
                    $absUrl = $baseUrl->resolve($relUrl);
                    $full_url = $absUrl->getURL();
                    if($full_url != $linkHref)
                    {
                        if($linkHref == '/')
                        {
                            $str = str_replace('href="' . $linkHref .'"', 'href="' . $full_url . '"', $str);
                            $str = str_replace("href='" . $linkHref ."'", 'href="' . $full_url . '"', $str);
                        }
                        else
                        {
                            $str = str_replace($linkHref, $full_url, $str);
                        }
                    }
                } catch (Exception $e) {
                    aiomatic_log_to_file('Unable to resolve (2) relative image link "' . $linkHref . '" against base "' . $url . '": ' . $e->getMessage());
                    continue;
                }
            }
        }
    }
    return $str;
}
function aiomatic_testPhantom()
{
    if(!function_exists('shell' . '_exec')) {
        return -1;
    }
    $disabled = explode(',', ini_get('disable_functions'));
    if(in_array('shell' . '_exec', $disabled))
    {
        return -2;
    }
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if (isset($aiomatic_Main_Settings['phantom_path']) && $aiomatic_Main_Settings['phantom_path'] != '') 
    {
        $phantomjs_comm = $aiomatic_Main_Settings['phantom_path'] . ' ';
    }
    else
    {
        $phantomjs_comm = 'phantomjs ';
    }
    if (isset($aiomatic_Main_Settings['enable_detailed_logging'])) {
        aiomatic_log_to_file('PhantomJS TEST command: ' . $phantomjs_comm);
    }
    $shefunc = trim(' s ') . trim(' h ') . 'ell' . '_exec';
    $cmdResult = $shefunc($phantomjs_comm . '-h 2>&1');
    if(stristr($cmdResult, 'Usage') !== false)
    {
        return 1;
    }
    return 0;
}
function aiomatic_get_extensions()
{
    if(!function_exists('is_plugin_active'))
    {
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    }
	$list = array(
        'amazon_s3'		=> array(
            'label' => esc_html__('Amazon S3 Storage For Images', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=aiomatic_admin_settings#tab-28'),
            'getlink' => 'https://coderevolution.ro/product/aiomatic-extension-amazon-s3-storage-for-images/',
            'icon' => 'https://i.ibb.co/6ZHGv5S/4923041-aws-icon.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Store royalty-free or AI-generated images on Amazon S3', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('aiomatic-extension-amazon-s3-images/aiomatic-extension-amazon-s3-images.php') ? '1' : '0',
        ),
        'amazon_api'		=> array(
            'label' => esc_html__('Amazon API', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=aiomatic_admin_settings#tab-2'),
            'getlink' => 'https://coderevolution.ro/product/aiomatic-extension-amazon-api/',
            'icon' => 'https://i.ibb.co/6JrS8L7/aws-api-gateway-icon.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Use the official Amazon API instead of using web scraping, to get Amazon product details', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('aiomatic-extension-amazon-api/aiomatic-extension-amazon-api.php') ? '1' : '0',
        ),
        'pdf_files'		=> array(
            'label' => esc_html__('PDF File Storage And Parsing', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=aiomatic_admin_settings'),
            'getlink' => 'https://coderevolution.ro/product/aiomatic-extension-pdf-file-storage-and-parsing/',
            'icon' => 'https://i.ibb.co/47bc0JF/download.jpg',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('PDF File Parsing And Storage Using OmniBlocks', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('aiomatic-extension-pdf-files/aiomatic-extension-pdf-files.php') ? '1' : '0',
        ),
        'fbomatic'		=> array(
            'label' => esc_html__('F-omatic Post Generator', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=fbomatic_facebook_panel'),
            'getlink' => 'https://1.envato.market/fbomatic',
            'icon' => 'https://s3.envato.com/files/361820408/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds two chatbot extensions & OmniBlocks for direct Facebook sharing of text and images', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('fbomatic-facebook-post-generator/fbomatic-facebook-post-generator.php') ? '1' : '0',
        ),
        'twitomatic'		=> array(
            'label' => esc_html__('Twitomatic Post Generator', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=twitomatic_twitter_panel'),
            'getlink' => 'https://1.envato.market/twitomatic',
            'icon' => 'https://s3.envato.com/files/222528386/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds a chatbot extension & OmniBlock for direct X (Twitter) sharing of content', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('twitomatic-twitter-post-generator/twitomatic-twitter-post-generator.php') ? '1' : '0',
        ),
        'youtubomatic'		=> array(
            'label' => esc_html__('Youtubomatic Post Generator', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=youtubomatic_community_panel'),
            'getlink' => 'https://1.envato.market/youtubomatic',
            'icon' => 'https://s3.envato.com/files/276765063/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds a chatbot extension & OmniBlock for direct YouTube Community sharing of content', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('youtubomatic-youtube-post-generator/youtubomatic-youtube-post-generator.php') ? '1' : '0',
        ),
        'linkedinomatic'		=> array(
            'label' => esc_html__('Linkedinomatic Post Generator', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=linkedinomatic_linkedin_panel'),
            'getlink' => 'https://1.envato.market/linkedinomatic',
            'icon' => 'https://s3.envato.com/files/244423310/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds a chatbot extension & OmniBlock for direct LinkedIn sharing of content', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('linkedinomatic-linkedin-post-generator/linkedinomatic-linkedin-post-generator.php') ? '1' : '0',
        ),
        'redditomatic'		=> array(
            'label' => esc_html__('Redditomatic Post Generator', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=redditomatic_reddit_panel'),
            'getlink' => 'https://1.envato.market/redditomatic',
            'icon' => 'https://s3.envato.com/files/224590839/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds a chatbot extension & OmniBlock for direct Reddit sharing of content', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('redditomatic-reddit-post-generator/redditomatic-reddit-post-generator.php') ? '1' : '0',
        ),
        'instamatic'		=> array(
            'label' => esc_html__('iMediamatic Instagram Bot', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=instamatic_admin_settings'),
            'getlink' => 'https://1.envato.market/instamatic',
            'icon' => 'https://s3.envato.com/files/223152623/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds a chatbot extension & OmniBlock for direct Instagram sharing of content', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('instamatic-instagram-post-generator/instamatic-instagram-post-generator.php') ? '1' : '0',
        ),
        'pinterestomatic'		=> array(
            'label' => esc_html__('Pinterestomatic Post Generator', 'aiomatic-automatic-ai-content-writer'),
            'link' => admin_url('admin.php?page=pinterestomatic_admin_settings'),
            'getlink' => 'https://1.envato.market/pinterestomatic',
            'icon' => 'https://s3.envato.com/files/223667806/avatar2.png',
            'extra_class' => '',
            'pro' => false,
            'description' => esc_html__('Adds a chatbot extension & OmniBlock for direct Pinterest sharing of content', 'aiomatic-automatic-ai-content-writer'),
            'enabled' => is_plugin_active('pinterestomatic-pinterest-post-generator/pinterestomatic-pinterest-post-generator.php') ? '1' : '0',
        )
	);
    $list[ 'new_extension' ] = array(
        'label'						=> esc_html__( 'Add new Extensions', 'aiomatic-automatic-ai-content-writer' ),
        'link' 						=> 'https://coderevolution.ro/product-category/aiomatic/',
        'getlink' 					=> 'https://coderevolution.ro/product-category/aiomatic/',
        'icon'						=> 'https://i.ibb.co/bdpSQhT/392530-add-create-cross-new-plus-icon.png',
        'extra_class' 		        => 'aiomatic-new-extension-box',
        'description'			    => '',
        'enabled'					=> 1,
    );
    return $list;
}
function aiomatic_replace_omniblocks_data($prompt, $current_keyword, $kiwis, $block_results)
{
    $matches = array();
    $i = 0;
    preg_match_all('~%regex\(\s*\"([^"]+?)\s*"\s*[,;]\s*\"([^"]*)\"\s*(?:[,;]\s*\"([^"]*?)\s*\")?(?:[,;]\s*\"([^"]*?)\s*\")?(?:[,;]\s*\"([^"]*?)\s*\")?\)%~si', $prompt, $matches);
    if (is_array($matches) && count($matches) && is_array($matches[0])) {
        for($i = 0; $i < count($matches[0]); $i++)
        {
            if (isset($matches[0][$i])) $fullmatch = $matches[0][$i];
            if (isset($matches[1][$i])) $search_in = aiomatic_replace_omniblocks_data($matches[1][$i], $current_keyword, $kiwis, $block_results);
            if (isset($matches[2][$i])) $matchpattern = $matches[2][$i];
            if (isset($matches[3][$i])) $element = $matches[3][$i];
            if (isset($matches[4][$i])) $delimeter = $matches[4][$i];if (isset($matches[5][$i])) $counter = $matches[5][$i];
            if (isset($matchpattern)) {
               if (preg_match('<^[\/#%+~[\]{}][\s\S]*[\/#%+~[\]{}]$>', $matchpattern, $z)) {
                  $ret = preg_match_all($matchpattern, $search_in, $submatches, PREG_PATTERN_ORDER);
               }
               else {
                  $ret = preg_match_all('~'.$matchpattern.'~si', $search_in, $submatches, PREG_PATTERN_ORDER);
               }
            }
            if (isset($submatches)) {
               if (is_array($submatches)) {
                  $empty_elements = array_keys($submatches[0], "");
                  foreach ($empty_elements as $e) {
                     unset($submatches[0][$e]);
                  }
                  $submatches[0] = array_unique($submatches[0]);
                  if (!is_numeric($element)) {
                     $element = 0;
                  }if (!is_numeric($counter)) {
                     $counter = 0;
                  }
                  if(isset($submatches[(int)($element)]))
                  {
                      $matched = $submatches[(int)($element)];
                  }
                  else
                  {
                      $matched = '';
                  }
                  $matched = array_unique((array)$matched);
                  if (empty($delimeter) || $delimeter == 'null') {
                     if (isset($matched[$counter])) $matched = $matched[$counter];
                  }
                  else {
                     $matched = implode($delimeter, $matched);
                  }
                  if (empty($matched)) {
                     $prompt = str_replace($fullmatch, '', $prompt);
                  } else {
                     $prompt = str_replace($fullmatch, $matched, $prompt);
                  }
               }
            }
        }
    }
    preg_match_all('~%regextext\(\s*\"([^"]+?)\s*"\s*,\s*\"([^"]*)\"\s*(?:,\s*\"([^"]*?)\s*\")?(?:,\s*\"([^"]*?)\s*\")?(?:,\s*\"([^"]*?)\s*\")?\)%~si', $prompt, $matches);
    if (is_array($matches) && count($matches) && is_array($matches[0])) {
        for($i = 0; $i < count($matches[0]); $i++)
        {
            if (isset($matches[0][$i])) $fullmatch = $matches[0][$i];
            if (isset($matches[1][$i])) $search_in = aiomatic_replace_omniblocks_data($matches[1][$i], $current_keyword, $kiwis, $block_results);
            if (isset($matches[2][$i])) $matchpattern = $matches[2][$i];
            if (isset($matches[3][$i])) $element = $matches[3][$i];
            if (isset($matches[4][$i])) $delimeter = $matches[4][$i];if (isset($matches[5][$i])) $counter = $matches[5][$i];
            $search_in = strip_tags($search_in, '<p><br>');
            $search_in1 = preg_replace("/<p[^>]*?>/", "", $search_in);
            if($search_in1 !== null)
            {
                $search_in = $search_in1;
            }
            $search_in = str_replace("</p>", "<br />", $search_in);
            $search_in1 = preg_replace('/\<br(\s*)?\/?\>/i', "\r\n\r\n", $search_in);
            if($search_in1 !== null)
            {
                $search_in = $search_in1;
            }
            $search_in1 = preg_replace('/^(?:\r|\n|\r\n)+/', '', $search_in);
            if($search_in1 !== null)
            {
                $search_in = $search_in1;
            }
            if (isset($matchpattern)) {
               if (preg_match('<^[\/#%+~[\]{}][\s\S]*[\/#%+~[\]{}]$>', $matchpattern, $z)) {
                  $ret = preg_match_all($matchpattern, $search_in, $submatches, PREG_PATTERN_ORDER);
               }
               else {
                  $ret = preg_match_all('~'.$matchpattern.'~si', $search_in, $submatches, PREG_PATTERN_ORDER);
               }
            }
            if (isset($submatches)) {
               if (is_array($submatches)) {
                  $empty_elements = array_keys($submatches[0], "");
                  foreach ($empty_elements as $e) {
                     unset($submatches[0][$e]);
                  }
                  $submatches[0] = array_unique($submatches[0]);
                  if (!is_numeric($element)) {
                     $element = 0;
                  }if (!is_numeric($counter)) {
                     $counter = 0;
                  }
                  if(isset($submatches[(int)($element)]))
                  {
                      $matched = $submatches[(int)($element)];
                  }
                  else
                  {
                      $matched = '';
                  }
                  $matched = array_unique((array)$matched);
                  if (empty($delimeter) || $delimeter == 'null') {
                     if (isset($matched[$counter])) $matched = $matched[$counter];
                  }
                  else {
                     $matched = implode($delimeter, $matched);
                  }
                  if (empty($matched)) {
                     $prompt = str_replace($fullmatch, '', $prompt);
                  } else {
                     $prompt = str_replace($fullmatch, $matched, $prompt);
                  }
               }
            }
        }
    }
    $spintax = new Aiomatic_Spintax();
    $prompt = $spintax->process($prompt);
    $pcxxx = explode('<!- template ->', $prompt);
    $prompt = $pcxxx[array_rand($pcxxx)];
    $prompt = str_replace('%%random_sentence%%', aiomatic_random_sentence_generator(), $prompt);
    $prompt = str_replace('%%random_sentence2%%', aiomatic_random_sentence_generator(false), $prompt);
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if (isset($aiomatic_Main_Settings['custom_html'])) {
        $xspintax = html_entity_decode($aiomatic_Main_Settings['custom_html']);
        $xspintax = $spintax->process($xspintax);
        $prompt = str_replace('%%custom_html%%', $xspintax, $prompt);
    }
    else
    {
        $prompt = str_replace('%%custom_html%%', '', $prompt);
    }
    if (isset($aiomatic_Main_Settings['custom_html2'])) {
        $xspintax2 = html_entity_decode($aiomatic_Main_Settings['custom_html2']);
        $xspintax2 = $spintax->process($xspintax2);
        $prompt = str_replace('%%custom_html2%%', $xspintax2, $prompt);
    }
    else
    {
        $prompt = str_replace('%%custom_html2%%', '', $prompt);
    }
    $prompt = str_replace('%%keyword%%', $current_keyword, $prompt);
    if(is_array($kiwis))
    {
        foreach($kiwis as $kws => $data)
        {
            $expdata = explode(',', $data);
            $expdata = trim($expdata[array_rand($expdata)]);
            $prompt = str_replace('%%' . $kws . '%%', $expdata, $prompt);
        }
    }
    if(is_array($block_results))
    {
        foreach($block_results as $blid => $block)
        {
            if(is_array($block) && isset($block[1]))
            {
                if(isset($block[1]))
                {
                    if(is_string($block[1]))
                    {
                        $prompt = str_replace('%%output_' . $blid . '%%', $block[1], $prompt);
                        $prompt = str_replace('%%output-' . $blid . '%%', $block[1], $prompt);
                    }
                }
                else
                {
                    $prompt = str_replace('%%output_' . $blid . '%%', '', $prompt);
                    $prompt = str_replace('%%output-' . $blid . '%%', '', $prompt);
                }
                if($block[0] == 'ai_text')
                {
                    $prompt = str_replace('%%ai_text_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'ai_text_foreach')
                {
                    $prompt = str_replace('%%ai_text_foreach_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'dalle_ai_image')
                {
                    $prompt = str_replace('%%dalle_image_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'royalty_image')
                {
                    $prompt = str_replace('%%free_image_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'stable_ai_image')
                {
                    $prompt = str_replace('%%stability_image_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'stable_ai_video')
                {
                    $prompt = str_replace('%%stability_video_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'nlp_entities_neuron')
                {
                    $prompt = str_replace('%%entities_title_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%entities_description_' . $blid . '%%', $block[2], $prompt);
                    $prompt = str_replace('%%entities_h1_' . $blid . '%%', $block[3], $prompt);
                    $prompt = str_replace('%%entities_h2_' . $blid . '%%', $block[4], $prompt);
                    $prompt = str_replace('%%entities_content_basic_' . $blid . '%%', $block[5], $prompt);
                    $prompt = str_replace('%%entities_content_basic_with_ranges_' . $blid . '%%', $block[6], $prompt);
                    $prompt = str_replace('%%entities_content_extended_' . $blid . '%%', $block[7], $prompt);
                    $prompt = str_replace('%%entities_content_extended_with_ranges_' . $blid . '%%', $block[8], $prompt);
                }
                elseif($block[0] == 'nlp_entities')
                {
                    $prompt = str_replace('%%entities_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%entities_details_json_' . $blid . '%%', $block[2], $prompt);
                }
                elseif($block[0] == 'webhook_fire')
                {
                    if(is_array($block[1]))
                    {
                        foreach($block[1] as $wkey => $wdata)
                        {
                            $prompt = str_replace('%%webhook_data_' . $blid . '_' . $wkey . '%%', $wdata, $prompt);
                        }
                        $prompt = str_replace('%%webhook_data_' . $blid . '%%', json_encode($block[1]), $prompt);
                    }
                    else
                    {
                        $prompt = str_replace('%%webhook_data_' . $blid . '%%', $block[1], $prompt);
                    }
                }
                elseif($block[0] == 'crawl_sites')
                {
                    $prompt = str_replace('%%scraped_content_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%scraped_content_plain_' . $blid . '%%', wp_strip_all_tags($block[1]), $prompt);
                }
                elseif($block[0] == 'crawl_rss')
                {
                    $prompt = str_replace('%%rss_content_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'google_search')
                {
                    $prompt = str_replace('%%search_result_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'text_translate')
                {
                    $prompt = str_replace('%%translated_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'text_spinner')
                {
                    $prompt = str_replace('%%spun_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'embeddings')
                {
                    $prompt = str_replace('%%embeddings_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'youtube_video')
                {
                    $prompt = str_replace('%%video_url_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%video_embed_' . $blid . '%%', $block[2], $prompt);
                }
                elseif($block[0] == 'post_import')
                {
                    $prompt = str_replace('%%post_id_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%post_url_' . $blid . '%%', $block[2], $prompt);
                    $prompt = str_replace('%%post_title_' . $blid . '%%', $block[3], $prompt);
                    $prompt = str_replace('%%post_content_' . $blid . '%%', $block[4], $prompt);
                    $prompt = str_replace('%%post_excerpt_' . $blid . '%%', $block[5], $prompt);
                    $prompt = str_replace('%%post_categories_' . $blid . '%%', $block[6], $prompt);
                    $prompt = str_replace('%%post_tags_' . $blid . '%%', $block[7], $prompt);
                    $prompt = str_replace('%%post_author_' . $blid . '%%', $block[8], $prompt);
                    $prompt = str_replace('%%post_date_' . $blid . '%%', $block[9], $prompt);
                    $prompt = str_replace('%%post_status_' . $blid . '%%', $block[10], $prompt);
                    $prompt = str_replace('%%post_type_' . $blid . '%%', $block[11], $prompt);
                    $prompt = str_replace('%%post_image_' . $blid . '%%', $block[12], $prompt);
                }
                elseif($block[0] == 'youtube_caption')
                {
                    $prompt = str_replace('%%video_caption_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%video_title_' . $blid . '%%', $block[2], $prompt);
                    $prompt = str_replace('%%video_description_' . $blid . '%%', $block[3], $prompt);
                    $prompt = str_replace('%%video_thumb_' . $blid . '%%', $block[4], $prompt);
                }
                elseif($block[0] == 'amazon_product')
                {
                    $prompt = str_replace('%%product_title_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%product_description_' . $blid . '%%', $block[2], $prompt);
                    $prompt = str_replace('%%product_url_' . $blid . '%%', $block[3], $prompt);
                    $prompt = str_replace('%%product_price_' . $blid . '%%', $block[4], $prompt);
                    $prompt = str_replace('%%product_list_price_' . $blid . '%%', $block[5], $prompt);
                    $prompt = str_replace('%%product_image_' . $blid . '%%', $block[6], $prompt);
                    $prompt = str_replace('%%product_cart_url_' . $blid . '%%', $block[7], $prompt);
                    $prompt = str_replace('%%product_images_urls_' . $blid . '%%', $block[8], $prompt);
                    $prompt = str_replace('%%product_images_' . $blid . '%%', $block[9], $prompt);
                    $prompt = str_replace('%%product_reviews_' . $blid . '%%', $block[10], $prompt);
                    $prompt = str_replace('%%product_reviews_' . $blid . '%%', $block[10], $prompt);
                    //new
                    $prompt = str_replace('%%product_score_' . $blid . '%%', $block[11], $prompt);
                    $prompt = str_replace('%%product_language_' . $blid . '%%', $block[12], $prompt);
                    $prompt = str_replace('%%product_edition_' . $blid . '%%', $block[13], $prompt);
                    $prompt = str_replace('%%product_pages_count_' . $blid . '%%', $block[14], $prompt);
                    $prompt = str_replace('%%product_publication_date_' . $blid . '%%', $block[15], $prompt);
                    $prompt = str_replace('%%product_contributors_' . $blid . '%%', $block[16], $prompt);
                    $prompt = str_replace('%%product_manufacturer_' . $blid . '%%', $block[17], $prompt);
                    $prompt = str_replace('%%product_binding_' . $blid . '%%', $block[18], $prompt);
                    $prompt = str_replace('%%product_product_group_' . $blid . '%%', $block[19], $prompt);
                    $prompt = str_replace('%%product_rating_' . $blid . '%%', $block[20], $prompt);
                    $prompt = str_replace('%%product_eans_' . $blid . '%%', $block[21], $prompt);
                    $prompt = str_replace('%%product_part_no_' . $blid . '%%', $block[22], $prompt);
                    $prompt = str_replace('%%product_model_' . $blid . '%%', $block[23], $prompt);
                    $prompt = str_replace('%%product_warranty_' . $blid . '%%', $block[24], $prompt);
                    $prompt = str_replace('%%product_color_' . $blid . '%%', $block[25], $prompt);
                    $prompt = str_replace('%%product_is_adult_' . $blid . '%%', $block[26], $prompt);
                    $prompt = str_replace('%%product_dimensions_' . $blid . '%%', $block[27], $prompt);
                    $prompt = str_replace('%%product_date_' . $blid . '%%', $block[28], $prompt);
                    $prompt = str_replace('%%product_size_' . $blid . '%%', $block[29], $prompt);
                    $prompt = str_replace('%%product_unit_count_' . $blid . '%%', $block[30], $prompt);
                }
                elseif($block[0] == 'amazon_listing')
                {
                    $prompt = str_replace('%%product_listing_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'load_file')
                {
                    if(is_array($block[1]))
                    {
                        preg_match_all('#%%xlsx_' . preg_quote($blid) . '_(\d+)_(\d+)%%#i', $prompt, $mmatch);
                        if(isset($mmatch[1][0]))
                        {
                            $cnt = 0;
                            for($cnt = 0; $cnt < count($mmatch[1]); $cnt++)
                            {
                                if(isset($mmatch[2][$cnt]))
                                {
                                    if(isset($block[1][$mmatch[1][$cnt]-1][$mmatch[2][$cnt]-1]))
                                    {
                                        $prompt = str_replace('%%xlsx_' . $blid . '_' . $mmatch[1][$cnt] . '_' . $mmatch[2][$cnt] . '%%', $block[1][$mmatch[1][$cnt]-1][$mmatch[2][$cnt]-1], $prompt);
                                    }
                                }
                                $prompt = str_replace('%%xlsx_' . $blid . '_' . $mmatch[1][$cnt] . '_' . $mmatch[2][$cnt] . '%%', '', $prompt);
                            }
                        }
                        preg_match_all('#%%xlsx_' . preg_quote($blid) . '_row_(\d+)%%#i', $prompt, $mmatch);
                        if(isset($mmatch[1][0]))
                        {
                            $cnt = 0;
                            for($cnt = 0; $cnt < count($mmatch[1]); $cnt++)
                            {
                                if(isset($block[1][$mmatch[1][$cnt]-1][0]))
                                {
                                    $row_data = '';
                                    foreach($block[1][$mmatch[1][$cnt]-1] as $rowme)
                                    {
                                        $row_data .= $rowme . ' ';
                                    }
                                    $prompt = str_replace('%%xlsx_' . $blid . '_row_' . $mmatch[1][$cnt] . '%%', $row_data, $prompt);
                                }
                                $prompt = str_replace('%%xlsx_' . $blid . '_row_' . $mmatch[1][$cnt] . '%%', '', $prompt);
                            }
                        }
                        preg_match_all('#%%xlsx_' . preg_quote($blid) . '_column_(\d+)%%#i', $prompt, $mmatch);
                        if(isset($mmatch[1][0]))
                        {
                            $cnt = 0;
                            for($cnt = 0; $cnt < count($mmatch[1]); $cnt++)
                            {
                                if(isset($block[1][0][$mmatch[1][$cnt]-1]))
                                {
                                    $column_data = '';
                                    for($xm = 0; $xm < count($block[1]); $xm++)
                                    {
                                        if(isset($block[1][$xm][$mmatch[1][$cnt]-1]))
                                        {
                                            $column_data .= $block[1][$xm][$mmatch[1][$cnt]-1] . ' ';
                                        }
                                    }
                                    $prompt = str_replace('%%xlsx_' . $blid . '_column_' . $mmatch[1][$cnt] . '%%', $column_data, $prompt);
                                }
                                $prompt = str_replace('%%xlsx_' . $blid . '_column_' . $mmatch[1][$cnt] . '%%', '', $prompt);
                            }
                        }
                        preg_match_all('#%%xlsx_' . preg_quote($blid) . '_row_random%%#i', $prompt, $mmatch);
                        if(isset($mmatch[0][0]))
                        {
                            $row_data = '';
                            $init_arr = $block[1];
                            while($row_data == '' && !empty($init_arr))
                            {
                                $cnt = array_rand($init_arr);
                                if(isset($init_arr[$cnt][0]))
                                {
                                    foreach($init_arr[$cnt] as $rowme)
                                    {
                                        $row_data .= $rowme . ' ';
                                    }
                                }
                                unset($init_arr[$cnt]);
                            }
                            $prompt = str_replace('%%xlsx_' . $blid . '_row_random%%', $row_data, $prompt);
                        }
                        preg_match_all('#%%xlsx_' . preg_quote($blid) . '_row_random_check%%#i', $prompt, $mmatch);
                        if(isset($mmatch[0][0]))
                        {
                            $row_data = '';
                            $init_arr = $block[1];
                            while($row_data == '' && !empty($init_arr))
                            {
                                $cnt = array_rand($init_arr);
                                if(isset($init_arr[$cnt][0]))
                                {
                                    foreach($init_arr[$cnt] as $rowme)
                                    {
                                        $row_data .= $rowme . ' ';
                                    }
                                }
                                unset($init_arr[$cnt]);
                                if($row_data !== '')
                                {
                                    $post_types = get_post_types(array('public' => true), 'names');
                                    $axargs = array(
                                        'post_type' => $post_types,
                                        'post_status' => 'publish',
                                        'posts_per_page' => -1,
                                        'title' => $row_data,
                                        'fields' => 'ids',
                                    );
                                    $zsposts = get_posts($axargs);
                                    if (!empty($zsposts)) {
                                        $row_data = '';
                                    }
                                }
                            }
                            $prompt = str_replace('%%xlsx_' . $blid . '_row_random_check%%', $row_data, $prompt);
                        }
                        $prompt = str_replace('%%file_' . $blid . '%%', '', $prompt);
                    }
                    else
                    {
                        $prompt = str_replace('%%file_' . $blid . '%%', $block[1], $prompt);
                    }
                }
                elseif($block[0] == 'save_post')
                {
                    $prompt = str_replace('%%created_post_id_' . $blid . '%%', $block[1], $prompt);
                    $prompt = str_replace('%%created_post_url_' . $blid . '%%', $block[2], $prompt);
                }
                elseif($block[0] == 'send_email')
                {
                }
                elseif($block[0] == 'send_facebook')
                {
                }
                elseif($block[0] == 'save_file')
                {
                }
                elseif($block[0] == 'send_twitter')
                {
                }
                elseif($block[0] == 'send_gmb')
                {
                }
                elseif($block[0] == 'send_community_youtube')
                {
                }
                elseif($block[0] == 'send_linkedin')
                {
                }
                elseif($block[0] == 'send_reddit')
                {
                }
                elseif($block[0] == 'send_webhook')
                {
                }
                elseif($block[0] == 'god_mode')
                {
                    $prompt = str_replace('%%god_mode_' . $blid . '%%', $block[1], $prompt);
                }
                elseif($block[0] == 'send_image_facebook')
                {
                }
                elseif($block[0] == 'send_image_instagram')
                {
                }
                elseif($block[0] == 'send_image_pinterest')
                {
                }
                else
                {
                    aiomatic_log_to_file('Unknown OmniBlock type submitted: ' . $block[0]);
                }
            }
        }
    }
    if ( is_user_logged_in() ) 
    {
        $user_id = get_current_user_id();
        if($user_id !== 0)
        {
            preg_match_all('#%%~([^!]*?)~%%#', $prompt, $matched_content);
            if(isset($matched_content[1][0]))
            {
                foreach($matched_content[1] as $mc)
                {
                    $post_custom_data = get_user_meta($user_id, $mc, true);
                    if($post_custom_data != '')
                    {
                        $prompt = str_replace('%%~' . $mc . '~%%', $post_custom_data, $prompt);
                    }
                    else
                    {
                        $prompt = str_replace('%%~' . $mc . '~%%', '', $prompt);
                    }
                }
            }
        }
        else
        {
            preg_match_all('#%%~([^!]*?)~%%#', $prompt, $matched_content);
            if(isset($matched_content[1][0]))
            {
                foreach($matched_content[1] as $mc)
                {
                    $prompt = str_replace('%%~' . $mc . '~%%', '', $prompt);
                }
            }
        }
    } 
    else 
    {
        preg_match_all('#%%~([^!]*?)~%%#', $prompt, $matched_content);
        if(isset($matched_content[1][0]))
        {
            foreach($matched_content[1] as $mc)
            {
                $prompt = str_replace('%%~' . $mc . '~%%', '', $prompt);
            }
        }
    }
    $prompt = preg_replace_callback('#%%random_image_url\[([^\]]*?)\]%%#', function ($matches) {
        $arv = array();
        $my_img = aiomatic_get_random_image_google($matches[1], 0, 0, '', $arv);
        return $my_img;
    }, $prompt);
    $prompt = preg_replace_callback('#%%random_image\[([^\]]*?)\](\[\d+\])?%%#', function ($matches) {
        if(isset($matches[2]))
        {
            $chance = trim($matches[2], '[]');
        }
        else
        {
            $chance = '';
        }
        $arv = array();
        $my_img = aiomatic_get_random_image_google($matches[1], 0, 0, $chance, $arv);
        return '<img src="' . $my_img . '">';
    }, $prompt);
    $prompt = preg_replace_callback('#%%random_video\[([^\]]*?)\](\[\d+\])?%%#', function ($matches) {
        if(isset($matches[2]))
        {
            $chance = trim($matches[2], '[]');
        }
        else
        {
            $chance = '';
        }
        $my_vid = aiomoatic_get_video($matches[1], $chance);
        return $my_vid;
    }, $prompt);
    $prompt = str_replace('%%current_date_time%%', date('Y/m/d H:i:s'), $prompt);
    $prompt = aiomatic_replaceSynergyShortcodes($prompt);
    $prompt = apply_filters('aiomatic_replace_aicontent_shortcode', $prompt);
    $prompt = do_shortcode($prompt);
    return trim($prompt);
}
function aiomatic_removeDuplicateNewLines($string) 
{
    return preg_replace("/[\r\n]+/", "\n", $string);
}
function aiomatic_scrape_page($url, $use_phantom, $type, $getname)
{
    require_once (dirname(__FILE__) . "/aiomatic-scraper.php");
    $custom_user_agent = aiomatic_get_random_user_agent();
    $custom_cookies = '';
    $use_proxy = '1';
    $user_pass = '';
    $phantom_wait = '';
    $request_delay = '';
    $scripter = '';
    $local_storage = '';
    $auto_captcha = '';
    $enable_adblock = '';
    $clickelement = '';
    $post_fields = '';
    $html_cont = false;
    $got_phantom = false;
    if($use_phantom == '1')
    {
        $html_cont = aiomatic_get_page_PhantomJS($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, $phantom_wait, $request_delay, $scripter, $local_storage);
        if($html_cont !== false)
        {
            $got_phantom = true;
        }
    }
    elseif($use_phantom == '2')
    {
        $html_cont = aiomatic_get_page_Puppeteer($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, $phantom_wait, $request_delay, $scripter, $local_storage);
        if($html_cont !== false)
        {
            $got_phantom = true;
        }
    }
    elseif($use_phantom == '3')
    {
        $html_cont = aiomatic_get_page_Tor($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, $phantom_wait, $request_delay, $scripter, $local_storage);
        if($html_cont !== false)
        {
            $got_phantom = true;
        }
    }
    elseif($use_phantom == '4')
    {
        $html_cont = aiomatic_get_page_PuppeteerAPI($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, $phantom_wait, $request_delay, $scripter, $local_storage, $auto_captcha, $enable_adblock, $clickelement);
        if($html_cont !== false)
        {
            $got_phantom = true;
        }
    }
    elseif($use_phantom == '5')
    {
        $html_cont = aiomatic_get_page_TorAPI($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, $phantom_wait, $request_delay, $scripter, $local_storage, $auto_captcha, $enable_adblock, $clickelement);
        if($html_cont !== false)
        {
            $got_phantom = true;
        }
    }
    elseif($use_phantom == '6')
    {
        $html_cont = aiomatic_get_page_PhantomJSAPI($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, $phantom_wait, $request_delay, $scripter, $local_storage);
        if($html_cont !== false)
        {
            $got_phantom = true;
        }
    }
    if($got_phantom === false)
    {
        $html_cont = aiomatic_get_web_page($url, $custom_cookies, $custom_user_agent, $use_proxy, $user_pass, '', $post_fields, $request_delay);
    }
    if($html_cont === false)
    {
        aiomatic_log_to_file('Failed to scrape content for: ' . $url); 
        return false;
    }
    if($type == 'raw')
    {
        return $html_cont;
    }
    $ret_cont = '';
    if($getname == '' || $type == 'auto')
    {
        $extract = aiomatic_convert_readable_html($html_cont);
        if($extract == '' || !isset($extract[1]))
        {
            aiomatic_log_to_file('Empty string returned: ' . $url); 
            return false;
        }
        else
        {
            $ret_cont = $extract[1];
        }
    }
    else
    {
        $extractorstr = '';
        $list_getname = preg_split('/\r\n|\r|\n/', $getname);
        foreach($list_getname as $my_getname)
        {
            $extractorstr .= aiomatic_get_content($type, $my_getname, $html_cont, false, false);
            if(!empty($extractorstr))
            {
                $extractorstr .= ' ';
            }
        }
        $ret_cont = $extractorstr;
        if($ret_cont == '')
        {
            $extract = aiomatic_convert_readable_html($html_cont);
            if($extract != '')
            {
                $ret_cont = $extract[1];
            }
        }
    }
    return $ret_cont;
}
use fivefilters\Readability\Readability;
use fivefilters\Readability\Configuration;
function aiomatic_convert_readable_html($html_string) 
{
    try 
    {
        if (version_compare(PHP_VERSION, '7.3.0', '<'))
        {
            throw new Exception('PHP is older than 7.3');
        }
        require_once (dirname(__FILE__) . "/res/readability/ReadabilityExtension.php");
        if(!class_exists('\fivefilters\Readability\Readability'))
        {
            require_once (dirname(__FILE__) . '/res/readability/vendor/autoload.php');

            require_once (dirname(__FILE__) . "/res/readability/Readability.php");
            require_once (dirname(__FILE__) . "/res/readability/ParseException.php");
            require_once (dirname(__FILE__) . "/res/readability/Configuration.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/NodeUtility.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/NodeTrait.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMAttr.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMNodeList.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMCdataSection.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMCharacterData.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMComment.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMDocument.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMDocumentFragment.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMDocumentType.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMElement.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMEntity.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMEntityReference.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMNode.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMNotation.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMProcessingInstruction.php");
            require_once (dirname(__FILE__) . "/res/readability/Nodes/DOM/DOMText.php");
        }
        $readConf = new Configuration();
        $readConf->setSummonCthulhu(true);
        $readability = new Readability($readConf);
        $readability->parse($html_string);
        $return_me[0] = $readability->getTitle();
        $return_me[1] = $readability->getContent();
        if($return_me[0] == '' || $return_me[0] == null || $return_me[1] == '' || $return_me[1] == null)
        {
            throw new Exception('Content/title blank ' . print_r($return_me, true));
        }
        $return_me[1] = str_replace('</article>', '', $return_me[1]);
        $return_me[1] = str_replace('<article>', '', $return_me[1]);
        return $return_me;
    } catch (Exception $e) {
        try
        {
            require_once (dirname(__FILE__) . "/res/aiomatic-readability.php");
            $readability = new Readability2($html_string);
            $readability->debug = false;
            $readability->convertLinksToFootnotes = false;
            $result = $readability->init();
            if ($result) {
                $return_me[0] = $readability->getTitle()->innerHTML;
                $return_me[1] = $readability->getContent()->innerHTML;
                $return_me[1] = str_replace('</article>', '', $return_me[1]);
                $return_me[1] = str_replace('<article>', '', $return_me[1]);
                return $return_me;
            } else {
                return '';
            }
        }
        catch(Exception $e2)
        {
            aiomatic_log_to_file('Readability failed: ' . sprintf('Error processing text: %s', $e2->getMessage()));
            return '';
        }
    }
}
function aiomatic_increment(&$string)
{
    $last_char = substr($string, -1);
    $rest = substr($string, 0, -1);
    switch ($last_char) {
    case '':
        $next = 'a';
        break;
    case 'z':
        $next = 'A';
        break;
    case 'Z':
        $next = '0';
        break;
    case '9':
        aiomatic_increment($rest);
        $next = 'a';
        break;
    default:
        $next = ++$last_char;
    }
    $string = $rest . $next;
}

function aiomatic_insert_attachment_by($value) {
    global $wpdb;
    $wpdb->get_results("
        INSERT INTO " . $wpdb->prefix . "posts" . " (post_author, guid, post_title, post_mime_type, post_type, post_status, post_parent, post_date, post_date_gmt, post_modified, post_modified_gmt, post_content, post_excerpt, to_ping, pinged, post_content_filtered) 
        VALUES " . $value);
    return $wpdb->insert_id;
}
function aiomatic_get_formatted_value($url, $alt, $post_parent) {
    return "(77777, '" . $url . "', '" . str_replace("'", "", $alt) . "', 'image/jpeg', 'attachment', 'inherit', '" . $post_parent . "', now(), now(), now(), now(), '', '', '', '', '')";
}
function aiomatic_url_is_image( $url ) 
{
    $url = str_replace(' ', '%20', $url);
    if ( ! filter_var( $url, FILTER_VALIDATE_URL ) ) {
        return FALSE;
    }
    $ext = array( 'jpeg', 'jpg', 'gif', 'png', 'jpe', 'tif', 'tiff', 'svg', 'ico' , 'webp', 'dds', 'heic', 'psd', 'pspimage', 'tga', 'thm', 'yuv', 'ai', 'eps', 'php');
    $info = (array) pathinfo( parse_url( $url, PHP_URL_PATH ) );
    if(!isset( $info['extension'] ))
    {
        return true;
    }
    return isset( $info['extension'] )
        && in_array( strtolower( $info['extension'] ), $ext, TRUE );
}
function aiomatic_custom_vision_upload_dir( $dir ) {
    return array(
        'path'   => $dir['basedir'] . '/ai-vision-images',
        'url'    => $dir['baseurl'] . '/ai-vision-images',
        'subdir' => '/ai-vision-images',
    ) + $dir;
}
function aiomatic_fatal_clear_job($job_id)
{
    $error = error_get_last();
    if ($error !== null && ($error['type'] === E_ERROR || $error['type'] === E_PARSE || $error['type'] === E_CORE_ERROR || $error['type'] === E_COMPILE_ERROR || $error['type'] === E_USER_ERROR)) 
    {
        aiomatic_job_set_status_failed($job_id, 'Job halted because of PHP error!');
    }
}
function aiomatic_job_set_status_pending($job_id, $data = array())
{
    if(strstr($job_id, 'job_') !== false)
    {
        delete_transient("aiomatic_job_" . $job_id . "_status");
        set_transient("aiomatic_job_" . $job_id . "_status", array('status' => 'pending', 'data' => $data), 24 * 60 * 60);
    }
}
function aiomatic_job_set_status_completed($job_id, $data)
{
    delete_transient("aiomatic_job_" . $job_id . "_status");
    set_transient("aiomatic_job_" . $job_id . "_status", array('status' => 'completed', 'data' => $data), 24 * 60 * 60);
}
function aiomatic_job_set_status_failed($job_id, $error)
{
    delete_transient("aiomatic_job_" . $job_id . "_status");
    set_transient("aiomatic_job_" . $job_id . "_status", array('status' => 'failed', 'data' => $error), 24 * 60 * 60);
}
function aiomatic_job_get_status($job_id)
{
    $status = get_transient("aiomatic_job_" . $job_id . "_status");
    if(isset($status['status']) && $status['status'] == 'completed')
    {
        delete_transient("aiomatic_job_" . $job_id . "_status");
    }
    return $status;
}
function aiomatic_extract_text_chars($text, $getLast = true, $limit = 500) 
{
    $textLength = strlen($text);
    if ($textLength <= $limit) 
    {
        return $text;
    } 
    else 
    {
        if ($getLast) 
        {
            $last = substr($text, -$limit);
            return $last;
        }
        else
        {
            $first = substr($text, 0, $limit);
            return $first;
        }
    }
}
function aiomatic_extract_paragraph($html, $getLast = true, $maxChars = 500) 
{
    $text = trim(strip_tags($html));
    if ($getLast) 
    {
        if (preg_match_all('/<p[^>]*>(.*?)<\/p>/is', $html, $matches)) 
        {
            $lastParagraph = end($matches[1]);
            $lastParagraph = trim(strip_tags($lastParagraph));
            $lastParagraph = substr($lastParagraph, 0, $maxChars);
            return $lastParagraph;
        }
    } 
    else 
    {
        if (preg_match_all('/<p[^>]*>(.*?)<\/p>/is', $html, $matches)) 
        {
            $firstParagraph = reset($matches[1]);
            $firstParagraph = trim(strip_tags($firstParagraph));
            $firstParagraph = substr($firstParagraph, 0, $maxChars);
            return $firstParagraph;
        }
    }
    if ($getLast) 
    {
        return substr($text, -min($maxChars, strlen($text)));
    } 
    else 
    {
        return substr($text, 0, min($maxChars, strlen($text)));
    }
}
function aiomatic_format_function_params($params) 
{
    $formattedParams = array();
    if(!is_array($params))
    {
        $params = array($params);
    }
    foreach ($params as $key => $value) 
    {
        if (is_string($value)) {
            $formattedParams[] = "$key => '" . addslashes($value) . "'";
        } elseif (is_array($value)) {
            $formattedParams[] = "$key => [" . aiomatic_format_function_params($value) . "]";
        } elseif (is_object($value)) {
            $formattedParams[] = "$key => '" . json_encode($value) . "'";
        } elseif (is_null($value)) {
            $formattedParams[] = "$key => null";
        } elseif (is_bool($value)) {
            $formattedParams[] = $value ? "$key => true" : "$key => false";
        } else {
            $formattedParams[] = "$key => $value";
        }
    }
    return implode(', ', $formattedParams);
}
class Aiomatic_Query_Parameter implements JsonSerializable 
{
    public $name;
    public $description;
    public $type;
    public $required;
    
    public function __construct( $name, $description, $type = "string", $required = false ) 
    {
        if ( !preg_match('/^[a-zA-Z0-9_-]{1,64}$/', $name) ) 
        {
            throw new InvalidArgumentException( "Invalid function name." );
        }
        $this->name = $name;
        $this->description = $description;
        $this->type = $type;
        $this->required = $required;
    }
    #[\ReturnTypeWillChange]
    public function jsonSerialize() 
    {
        $retz = [
            'type' => $this->type,
            'description' => $this->description
        ];
        if($this->type === 'array')
        {
            $retz['items'] = array('type' => 'string');
        }
        return $retz;
    }
}
class Aiomatic_Query_Function implements JsonSerializable 
{
    public $name;
    public $description;
    public $parameters;
    public function __construct( $name, $description, $parameters = []) 
    {
        if ( !preg_match( '/^[a-zA-Z0-9_-]{1,64}$/', $name ) ) 
        {
            throw new InvalidArgumentException( "Invalid function name ($name). It must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64." );
        }
        foreach ( $parameters as $parameter ) 
        {
            if ( !( $parameter instanceof Aiomatic_Query_Parameter ) ) {
                throw new InvalidArgumentException( "Invalid parameter." );
            }
        }
        $this->name = $name;
        $this->description = $description;
        $this->parameters = $parameters;
    }
    #[\ReturnTypeWillChange]
    public function jsonSerialize() 
    {
        $params = [];
        foreach( $this->parameters as $parameter ) 
        {
            $params[ $parameter->name ] = $parameter;
        }
        $required = array_filter( $this->parameters, function( $param ) { return $param->required; } );
        $required = array_map( function( $param ) { return $param->name; }, $required );
        $json = [
            'name' => $this->name,
            'description' => $this->description,
            'parameters' => [
                'type' => 'object',
                'properties' => $params,
                'required' => $required
            ]
        ];
        return $json;
    }
}
function aiomatic_gen_uid($l=6){
    return substr(str_shuffle("0123456789abcdefghijklmnopqrstuvwxyz"), 0, $l);
}
function aiomatic_get_amazon_sorts()
{
    return array(
        'none' => esc_html__('None', 'amazomatic-amazon-post-generator') , 
        'Relevance' => esc_html__('Relevance', 'amazomatic-amazon-post-generator') , 
        'Price:LowToHigh' => esc_html__('Price:LowToHigh', 'amazomatic-amazon-post-generator') ,  
        'Price:HighToLow' => esc_html__('Price:HighToLow', 'amazomatic-amazon-post-generator') , 
        'NewestArrivals' => esc_html__('NewestArrivals', 'amazomatic-amazon-post-generator') ,
        'Featured' => esc_html__('Featured', 'amazomatic-amazon-post-generator') ,
        'AvgCustomerReviews' => esc_html__('AvgCustomerReviews', 'amazomatic-amazon-post-generator')
    );
}
function aiomatic_get_amazon_codes()
{
    return array(
        'com' => esc_html__('United States', 'amazomatic-amazon-post-generator') , 
        'co.uk' => esc_html__('United Kingdom', 'amazomatic-amazon-post-generator') , 
        'ae' => esc_html__('United Arab Emirates', 'amazomatic-amazon-post-generator') ,  
        'com.tr' => esc_html__('Turkey', 'amazomatic-amazon-post-generator') , 
        'es' => esc_html__('Spain', 'amazomatic-amazon-post-generator') ,
        'sg' => esc_html__('Singapore', 'amazomatic-amazon-post-generator') ,
        'com.mx' => esc_html__('Mexico', 'amazomatic-amazon-post-generator') , 
        'co.jp' => esc_html__('Japan', 'amazomatic-amazon-post-generator') , 
        'it' => esc_html__('Italy', 'amazomatic-amazon-post-generator') , 
        'ca' => esc_html__('Canada', 'amazomatic-amazon-post-generator') , 
        'de' => esc_html__('Germany', 'amazomatic-amazon-post-generator') , 
        'fr' => esc_html__('France', 'amazomatic-amazon-post-generator') ,  
        'com.br' => esc_html__('Brasil', 'amazomatic-amazon-post-generator') ,  
        'in' => esc_html__('India', 'amazomatic-amazon-post-generator') , 
        'com.au' => esc_html__('Australia', 'amazomatic-amazon-post-generator') , 
        'eg' => esc_html__('Egypt', 'amazomatic-amazon-post-generator') , 
        'pl' => esc_html__('Poland', 'amazomatic-amazon-post-generator') , 
        'sa' => esc_html__('Saudi Arabia', 'amazomatic-amazon-post-generator') , 
        'se' => esc_html__('Sweden', 'amazomatic-amazon-post-generator'), 
        'nl' => esc_html__('Netherlands', 'amazomatic-amazon-post-generator')
    );
}
function addCharacterToVisibleText($html, $character) {
    $dom = new DOMDocument();
    $internalErrors = libxml_use_internal_errors(true);
    $dom->loadHTML($html, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    libxml_use_internal_errors($internalErrors);
    $xpath = new DOMXPath($dom);
    $textNodes = $xpath->query('//text()');
    if($textNodes !== false)
    {
        foreach ($textNodes as $textNode) {
            $nodeValue = $textNode->nodeValue;
            $visibleText = trim($nodeValue);
            if (!empty($visibleText)) {
                $randomPosition = rand(0, 1);
                $newText = str_replace(' ', $character . ' ', $visibleText);
                if ($randomPosition == 0) {
                $newText = $character . $newText;
                } else {
                $newText .= $character;
                }
                $textNode->nodeValue = $newText;
            }
        }
    }
    $modifiedHtml = $dom->saveHTML();
    $modifiedHtml = preg_replace('~<(?:!DOCTYPE|/?(?:html|body))[^>]*>\s*~i', '', $modifiedHtml);
    if(!empty($modifiedHtml))
    {
        return $modifiedHtml;
    }
    return $html;
}
function aiomatic_compare_fill(&$measure,&$fill) {
	if (count($measure) != count($fill)) {
        while (count($fill) < count($measure) ) {
		    $fill = array_merge( $fill, array_values($fill) );
		}
		$fill = array_slice($fill, 0, count($measure));
	}
}

function aiomatic_my_user_by_rand( $ua ) {
    remove_action('pre_user_query', 'aiomatic_my_user_by_rand');
    $ua->query_orderby = str_replace( 'user_login ASC', 'RAND()', $ua->query_orderby );
}
function aiomatic_stringContainsArrayChars($string, $charsArray) {
    foreach ($charsArray as $char) {
        if (strpos($string, $char) !== false) {
            return true;
        }
    }
    return false;
}
function aiomatic_display_random_user(){
    add_action('pre_user_query', 'aiomatic_my_user_by_rand');
    $args = array(
      'orderby' => 'user_login', 'order' => 'ASC', 'number' => 1, 'role__in' => array( 'contributor','author','editor','administrator','super-admin' )
    );
    $user_query = new WP_User_Query( $args );
    $user_query->query();
    $results = $user_query->results;
    if(empty($results))
    {
        return false;
    }
    shuffle($results);
    return array_pop($results);
}
function aiomatic_make_unique($text, $characters, $percentage) 
{
    if(!is_array($characters) || empty($characters))
    {
        return $text;
    }
    $result = '';
    $htmlfounds = array();
    $pre_tags_matches = array();
    $pre_tags_matches_s = array();
    $conseqMatchs = array();
    $final_content_pre = aiomatic_replaceExcludes($text, $htmlfounds, $pre_tags_matches, $pre_tags_matches_s, $conseqMatchs);
    $textLength = strlen($final_content_pre);
    for ($i = 0; $i < $textLength; $i++) 
    {
        if ($final_content_pre[$i] == ' ' && rand(1, 100) <= $percentage) 
        {
            $character = $characters[array_rand($characters)];
            $randomPosition = rand(0, 1);
            if ($randomPosition == 0) {
                $result .= $character . ' ';
            } 
            else 
            {
                $result .= ' ' . $character;
            }
        } 
        else 
        {
            $result .= $final_content_pre[$i];
        }
    }
    if(empty($result))
    {
        return $text;
    }
    $result = aiomatic_restoreExcludes($result, $htmlfounds, $pre_tags_matches, $pre_tags_matches_s, $conseqMatchs);
    return $result;
}
function aiomatic_make_unique_HTML($text, $characters, $percentage) 
{
    if(!is_array($characters))
    {
        return $text;
    }
    $dom = new DOMDocument();
    $internalErrors = libxml_use_internal_errors(true);
    $dom->loadHTML($text, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    libxml_use_internal_errors($internalErrors);
    $xpath = new DOMXPath($dom);
    $textNodes = $xpath->query('//text()');
    if($textNodes !== false)
    {
        foreach ($textNodes as $textNode) 
        {
            $result = '';
            $nodeValue = $textNode->nodeValue;
            $visibleText = trim($nodeValue);
            if (!empty($visibleText)) 
            {
                $textLength = strlen($visibleText);
                for ($i = 0; $i < $textLength; $i++) 
                {
                    if ($visibleText[$i] == ' ' && rand(1, 100) <= $percentage) 
                    {
                        $randomPosition = rand(0, 1);
                        $character = $characters[array_rand($characters)];
                        if ($randomPosition == 0) 
                        {
                            $result .= $character . ' ';
                        } 
                        else 
                        {
                            $result .= ' ' . $character;
                        }
                    }
                    else 
                    {
                        $result .= $text[$i];
                    }
                }
                $textNode->nodeValue = $result;
            }
        }
    }
    $modifiedHtml = $dom->saveHTML();
    $modifiedHtml = preg_replace('~<(?:!DOCTYPE|/?(?:html|body))[^>]*>\s*~i', '', $modifiedHtml);
    if(!empty($modifiedHtml))
    {
        return $modifiedHtml;
    }
    return $text;
}
function aiomatic_randomName() {
    $firstname = array(
        'Johnathon',
        'Anthony',
        'Erasmo',
        'Raleigh',
        'Nancie',
        'Tama',
        'Camellia',
        'Augustine',
        'Christeen',
        'Luz',
        'Diego',
        'Lyndia',
        'Thomas',
        'Georgianna',
        'Leigha',
        'Alejandro',
        'Marquis',
        'Joan',
        'Stephania',
        'Elroy',
        'Zonia',
        'Buffy',
        'Sharie',
        'Blythe',
        'Gaylene',
        'Elida',
        'Randy',
        'Margarete',
        'Margarett',
        'Dion',
        'Tomi',
        'Arden',
        'Clora',
        'Laine',
        'Becki',
        'Margherita',
        'Bong',
        'Jeanice',
        'Qiana',
        'Lawanda',
        'Rebecka',
        'Maribel',
        'Tami',
        'Yuri',
        'Michele',
        'Rubi',
        'Larisa',
        'Lloyd',
        'Tyisha',
        'Samatha',
    );

    $lastname = array(
        'Mischke',
        'Serna',
        'Pingree',
        'Mcnaught',
        'Pepper',
        'Schildgen',
        'Mongold',
        'Wrona',
        'Geddes',
        'Lanz',
        'Fetzer',
        'Schroeder',
        'Block',
        'Mayoral',
        'Fleishman',
        'Roberie',
        'Latson',
        'Lupo',
        'Motsinger',
        'Drews',
        'Coby',
        'Redner',
        'Culton',
        'Howe',
        'Stoval',
        'Michaud',
        'Mote',
        'Menjivar',
        'Wiers',
        'Paris',
        'Grisby',
        'Noren',
        'Damron',
        'Kazmierczak',
        'Haslett',
        'Guillemette',
        'Buresh',
        'Center',
        'Kucera',
        'Catt',
        'Badon',
        'Grumbles',
        'Antes',
        'Byron',
        'Volkman',
        'Klemp',
        'Pekar',
        'Pecora',
        'Schewe',
        'Ramage',
    );

    $name = $firstname[rand ( 0 , count($firstname) -1)];
    $name .= ' ';
    $name .= $lastname[rand ( 0 , count($lastname) -1)];

    return $name;
}
function aiomatic_get_transients_by_regex($pattern, $users_per_page, &$trnsi_cnt) 
{
    $transi_count = 0;
    global $wpdb;
    $transients = $wpdb->get_col("
        SELECT option_name
        FROM $wpdb->options
        WHERE option_name REGEXP '^_transient_" . $wpdb->esc_like($pattern) . "'
    ");

    $transient_values = array();
    foreach ($transients as $transient) 
    {
        $transient_name = str_replace('_transient_', '', $transient);
        $transient_value = get_transient($transient_name);
        if ($transient_value !== false) 
        {
            if(count($transient_values) < $users_per_page)
            {
                $transient_values[$transient_name] = $transient_value;
            }
            $transi_count++;
        }
    }
    $trnsi_cnt = $transi_count;
    return $transient_values;
}
function aiomatic_log_exec_time($context)
{
    ob_start();
    phpinfo();
    $phpinfoOutput = ob_get_clean();
    $timeoutPattern = '#max_execution_time<\/td><td class="v">([^<]+)<#i';
    if (preg_match_all($timeoutPattern, $phpinfoOutput, $matches)) {
        $maxExecutionTime = intval($matches[1][0]);
        aiomatic_log_to_file("[" . $context . "] Starting execution, setting max_execution_time: " . $maxExecutionTime . " seconds");
    } else {
        aiomatic_log_to_file("[" . $context . "] Max Execution Time (ini_set) not found");
    }
}
function aiomatic_removeUrlParameter($url, $paramKey) {
    $parsedUrl = parse_url($url);

    if (isset($parsedUrl['query'])) 
    {
        parse_str($parsedUrl['query'], $params);
        if(isset($params[$paramKey]))
        {
            unset($params[$paramKey]);
        }
        $query = http_build_query($params);
        $updatedUrl = $parsedUrl['path'] . '?' . $query;
    } 
    else 
    {
        $updatedUrl = $url;
    }

    return $updatedUrl;
}
function aiomatic_get_the_user_ip() {

    if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) 
    {
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    } elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) 
    {
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    } else 
    {
        $ip = $_SERVER['REMOTE_ADDR'];
    }
    return apply_filters( 'aiomatic_get_ip', $ip );
}
function aiomatic_test_post_reponse() {

	$post_response = wp_safe_remote_post(
		'https://www.paypal.com/cgi-bin/webscr',
		array(
			'timeout'     => 60,
			'httpversion' => '1.1',
			'body'        => array(
				'cmd' => '_notify-validate',
			),
		)
	);


	if ( ! is_wp_error( $post_response ) && $post_response['response']['code'] >= 200 && $post_response['response']['code'] < 300 ) {
		return true;
	}

	return false;
}

function aiomatic_test_get_reponse() {

	$get_response = wp_safe_remote_get( 'https://woocommerce.com/wc-api/product-key-api?request=ping&network=' . ( is_multisite() ? '1' : '0' ) );

	if ( ! is_wp_error( $get_response ) && $get_response['response']['code'] >= 200 && $get_response['response']['code'] < 300 ) {
		return true;
	}

	return false;
}
function aiomatic_check_cron_status() {
	global $wp_version;

	if ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON ) {
		/* translators: 1: The name of the PHP constant that is set. */
		return new WP_Error( 'crontrol_info', sprintf( __( 'The %s constant is set to true. WP-Cron spawning is disabled.', 'wp-content-pilot' ), 'DISABLE_WP_CRON' ) );
	}

	if ( defined( 'ALTERNATE_WP_CRON' ) && ALTERNATE_WP_CRON ) {
		/* translators: 1: The name of the PHP constant that is set. */
		return new WP_Error( 'crontrol_info', sprintf( __( 'The %s constant is set to true.', 'wp-content-pilot' ), 'ALTERNATE_WP_CRON' ) );
	}

	$cached_status = get_transient( 'wpcp-cron-test-ok' );

	if ( $cached_status ) {
		return true;
	}

	$sslverify     = version_compare( $wp_version, 4.0, '<' );
	$doing_wp_cron = sprintf( '%.22F', microtime( true ) );

	$cron_request = apply_filters( 'cron_request', array(
		'url'  => site_url( 'wp-cron.php?doing_wp_cron=' . $doing_wp_cron ),
		'key'  => $doing_wp_cron,
		'args' => array(
			'timeout'   => 30,
			'blocking'  => true,
			'sslverify' => apply_filters( 'https_local_ssl_verify', $sslverify ),
		),
	) );

	$cron_request['args']['blocking'] = true;

	$result = wp_remote_post( $cron_request['url'], $cron_request['args'] );

	if ( is_wp_error( $result ) ) {
		return $result;
	} elseif ( wp_remote_retrieve_response_code( $result ) >= 300 ) {
		return new WP_Error( 'unexpected_http_response_code', sprintf(
		/* translators: 1: The HTTP response code. */
			__( 'Unexpected HTTP response code: %s', 'wp-content-pilot' ),
			intval( wp_remote_retrieve_response_code( $result ) )
		) );
	} else {
		set_transient( 'wpcp-cron-test-ok', 1, 3600 );

		return true;
	}

}
function aiomatic_let_to_num( $size ) {
    $l   = substr( $size, - 1 );
    $ret = substr( $size, 0, - 1 );
    switch ( strtoupper( $l ) ) {
        case 'P':
            $ret *= 1024;
        case 'T':
            $ret *= 1024;
        case 'G':
            $ret *= 1024;
        case 'M':
            $ret *= 1024;
        case 'K':
            $ret *= 1024;
    }

    return $ret;
}
function aiomatic_substr($prompt, $start, $len = null)
{
    if(function_exists('mb_substr'))
    {
        $prompt = mb_substr($prompt, $start, $len, 'UTF-8');
    }
    else
    {
        $prompt = substr($prompt, $start, $len);
    }
    return $prompt;
}
function aiomatic_seo_plugins_active()
{
    if (!function_exists('is_plugin_active')) {
        include_once(ABSPATH . 'wp-admin/includes/plugin.php');
    }
    $seo_plugin_activated = false;
    if(is_plugin_active('wordpress-seo/wp-seo.php')){
        $seo_plugin_activated = '_yoast_wpseo_metadesc';
    }
    elseif(is_plugin_active('all-in-one-seo-pack/all_in_one_seo_pack.php')){
        $seo_plugin_activated = '_aioseo_description';
    }
    elseif(is_plugin_active('seo-by-rank-math/rank-math.php')){
        $seo_plugin_activated = 'rank_math_description';
    }
    elseif(is_plugin_active('autodescription/autodescription.php')){
        $seo_plugin_activated = '_genesis_description';
    }
    return $seo_plugin_activated;
}
function aiomatic_save_seo_description($post_id, $description)
{
    global $wpdb;
    if(empty($description))
    {
        return;
    }
    $seo_plugin_activated = aiomatic_seo_plugins_active();
    if($seo_plugin_activated == '_yoast_wpseo_metadesc')
    {
        update_post_meta($post_id, $seo_plugin_activated, $description);
    }
    elseif($seo_plugin_activated == '_aioseo_description')
    {
        update_post_meta($post_id, $seo_plugin_activated, $description);
        $check = $wpdb->get_row($wpdb->prepare("SELECT * FROM ".$wpdb->prefix."aioseo_posts WHERE post_id=%d",$post_id));
        if($check)
        {
            $wpdb->update($wpdb->prefix.'aioseo_posts',array(
                'description' => $description
            ), array(
                'post_id' => $post_id
            ));
        }
        else{
            $wpdb->insert($wpdb->prefix.'aioseo_posts',array(
                'post_id' => $post_id,
                'description' => $description,
                'created' => date('Y-m-d H:i:s'),
                'updated' => date('Y-m-d H:i:s')
            ));
        }
    }
    elseif($seo_plugin_activated == 'rank_math_description')
    {
        update_post_meta($post_id, $seo_plugin_activated, $description);
    }
    elseif($seo_plugin_activated == '_genesis_description')
    {
        update_post_meta($post_id, $seo_plugin_activated, $description);
    }
    elseif($seo_plugin_activated == false)
    {
        $seo_plugin_activated = 'aiomatic_html_meta';
        update_post_meta($post_id, $seo_plugin_activated, $description);
    }
}
function aiomatic_change_post_status($post_id, $status){
    $current_post = get_post( $post_id, 'ARRAY_A' );
    $current_post['post_status'] = $status;
    remove_filter('content_save_pre', 'wp_filter_post_kses');
    remove_filter('content_filtered_save_pre', 'wp_filter_post_kses');remove_filter('title_save_pre', 'wp_filter_kses');
    wp_update_post($current_post);
    add_filter('content_save_pre', 'wp_filter_post_kses');
    add_filter('content_filtered_save_pre', 'wp_filter_post_kses');add_filter('title_save_pre', 'wp_filter_kses');
}
function aiomatic_save_term_seo_description($term_id, $description, $taxonomy)
{
    if(empty($description))
    {
        return;
    }
    $seo_plugin_activated = aiomatic_seo_plugins_active();
    if($seo_plugin_activated == '_yoast_wpseo_metadesc')
    {
        $yoast_options = get_option('wpseo_taxonomy_meta');
        $yoast_options[$taxonomy][$term_id]['wpseo_desc'] = $description;
        update_option('wpseo_taxonomy_meta', $yoast_options);
    }
    elseif($seo_plugin_activated == '_aioseo_description')
    {
        update_term_meta($term_id, '_aioseop_description', $description);
    }
    elseif($seo_plugin_activated == 'rank_math_description')
    {
        update_term_meta($term_id, 'rank_math_description', $description);
    }
    elseif($seo_plugin_activated == '_genesis_description')
    {
        update_term_meta($term_id, $seo_plugin_activated, $description);
    }
    elseif($seo_plugin_activated == false)
    {
        $seo_plugin_activated = 'aiomatic_html_meta';
        update_term_meta($term_id, $seo_plugin_activated, $description);
    }
}
function aiomatic_get_random_word($min = 4, $max = 10) 
{
    $word = array_merge(range('a', 'z'), range('A', 'Z'));
    shuffle($word);
    $len = rand($min, $max);
    return substr(implode($word), 0, $len);
}
function aiomatic_is_gutenberg_page() 
{
    if ( function_exists( 'is_gutenberg_page' ) &&
            is_gutenberg_page()
    ) 
    {
        return true;
    }
    $current_screen = get_current_screen();
    if ( method_exists( $current_screen, 'is_block_editor' ) &&
            $current_screen->is_block_editor()
    ) 
    {
        return true;
    }
    return false;
}
function aiomatic_get_api_service($token)
{
    if(aiomatic_is_aiomaticapi_key($token))
    {
        $api_service = 'AiomaticAPI';
    }
    else
    {
        $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
        if (aiomatic_check_if_azure($aiomatic_Main_Settings))
        {
            $api_service = 'Microsoft Azure OpenAI';
        }
        else
        {
            $api_service = 'OpenAI';
        }
    }
    return $api_service;
}
function aiomatic_check_is_elementor($postid)
{
    if (!function_exists('is_plugin_active')) {
        include_once(ABSPATH . 'wp-admin/includes/plugin.php');
    }
    if (!is_plugin_active( 'elementor/elementor.php' )) 
    {
		return false;
	}
    if(!isset(\Elementor\Plugin::$instance))
    {
        return false;
    }
    return \Elementor\Plugin::$instance->db->is_built_with_elementor($postid);
}
function aiomatic_upload_base64_image($base64_img, $title, $post_id)
{
    $upload_dir = wp_upload_dir();
    $upload_path = str_replace( '/', DIRECTORY_SEPARATOR, $upload_dir['path'] ) . DIRECTORY_SEPARATOR;
    $image_parts = explode(";base64,", $base64_img);
    $decoded = base64_decode($image_parts[1]);
    $filename = sanitize_title($title) . '.png';
    $hashed_filename = md5( $filename . microtime() ) . '_' . $filename;
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
        include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
        wp_filesystem($creds);
    }
    $ret = $wp_filesystem->put_contents( $upload_path . $hashed_filename, $decoded ); 
    if ($ret === FALSE) 
    {
        aiomatic_log_to_file('Failed to copy image locally ' . $upload_path . $hashed_filename);
        return false;
    }
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    require_once(ABSPATH . 'wp-admin/includes/media.php');
    require_once(ABSPATH . 'wp-admin/includes/file.php');
    $file             = array();
    $file['error']    = '';
    $file['tmp_name'] = $upload_path . $hashed_filename;
    $file['name']     = $hashed_filename;
    $file['type']     = 'image/png';
    $file['size']     = $wp_filesystem->size( $upload_path . $hashed_filename );
    $file_return = wp_handle_sideload($file, array( 'test_form' => false ));
    if(!isset($file_return['file']))
    {
        aiomatic_log_to_file('Failed to copy image file locally ' . $upload_path . $hashed_filename . ': ' . print_r($file_return, true));
        return false;
    }
    $filename = $file_return['file'];
    $attachment = array(
        'post_mime_type' => $file_return['type'],
        'post_title' => preg_replace('/\.[^.]+$/', '', basename($filename)),
        'post_content' => '',
        'post_status' => 'inherit',
        'guid' => $upload_dir['url'] . '/' . basename($filename)
    );
    $attach_id = wp_insert_attachment( $attachment, $filename, $post_id );
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    require_once(ABSPATH . 'wp-admin/includes/media.php');
    $attach_data = wp_generate_attachment_metadata($attach_id, $filename);
    wp_update_attachment_metadata($attach_id, $attach_data);
    return $attach_id;
}
function aiomatic_my_get_current_user_roles() {
    
    if( is_user_logged_in() ) {
  
      $user = wp_get_current_user();
  
      $roles = ( array ) $user->roles;
  
      return $roles;
  
    } else {
  
      return array();
  
    }
  
}
function aiomatic_my_get_current_user_subscriptions() {
    $levels = array();
    if(class_exists('Ihc_Db'))
    {
        if( is_user_logged_in() ) 
        {
            $current_user = wp_get_current_user();
            if(isset($current_user->ID))
            {
                $user_sub_data = Ihc_Db::get_user_levels($current_user->ID, true);
                if(is_array($user_sub_data))
                {
                    foreach($user_sub_data as $udata)
                    {
                        if(isset($udata['level_id']))
                        {
                            $levels[] = $udata['level_id'];
                        }
                    }
                }
            }  
        }
    }
    return $levels;
}
function aiomatic_get_assistant()
{
    $aiomatic_assistant_defaults = array("Write a paragraph on this" => array(
"0" => 'Write a paragraph on this topic:

%%selected_text%%
    
----
Written paragraph:
',
        "1" => 'text'
),

"Continue this text" => Array
    (
        "0" => 'Continue this text:

%%selected_text%%

----
Continued text:
',
"1" => 'text'
    ),

"Generate ideas on this" => Array
    (
        "0"=> 'Write a few ideas on that as bullet points:

%%selected_text%%

----
Generated ideas in bullet points:
',
"1" => 'text'
    ),

"Write an article about this" => Array
    (
        "0" => 'Write a complete article about this:

%%selected_text%%

----
Written article:
',
"1" => 'text'
    ),

"Turn this into an Ad" => Array
    (
"0" => 'Turn the following text into a creative advertisement:

%%selected_text%%

----
Advertisement:
',
"1" => 'text'
    ),

"Explain this to a 5 year old" => Array
    (
"0" => 'Explain this to a 5 years old kid:

%%selected_text%%

----
Explanation:
',
"1" => 'text'
    ),

"Find a matching quote for this" => Array
    (
"0" => 'Find a matching quote for the following text:

%%selected_text%%

----
Matching quote:
',
"1" => 'text'
    ),

"Generate a subtitle for this" => Array
    (
        "0" => 'Generate a title for this text:

%%selected_text%%

----
Title:
',
"1" => 'text'
    ),

"Generate a TL;DR of this" => Array
    (
        "0" => 'Write a TL;DR for this text:

%%selected_text%%

----
TL;DR:
',
"1" => 'text'
    ),

"Generate a Call to Action fo this" => Array
    (
        "0" => 'Generate a call to action about this:

%%selected_text%%

----
Call to action:
',
"1" => 'text'
    ),

"Summarize this" => Array
    (
        "0" => 'Summarize this text:

%%selected_text%%

----
Summary:
',
"1" => 'text'
    ),

"Expand this" => Array
    (
        "0" => 'Expand this text:

%%selected_text%%

----
Expanded text:
',
"1" => 'text'
    ),

"Make a bulleted list for this" => Array
    (
        "0" => 'Make a C for this:

%%selected_text%%

----
Bulleted list:
',
"1" => 'text'
    ),

"Rewrite this" => Array
    (
        "0" => 'Rewrite this text:

%%selected_text%%

----
Rewritten text:
',
"1" => 'text'
    ),

"Paraphrase this" => Array
    (
        "0" => 'Paraphrase this text:

%%selected_text%%

----
Paraphrased text:
',
"1" => 'text'
    ),

"Fix grammar of this" => Array
    (
        "0" => 'Fix grammar of this text:

%%selected_text%%

----
Text with fixed grammar:
',
"1" => 'text'
    ),

"Generate a question of this" => Array
    (
        "0" => 'Generate a question about this text:

%%selected_text%%

----
Question:
',
"1" => 'text'
    ),

"Convert this to passive voice" => Array
    (
        "0" => 'Convert this text to passive voice:

%%selected_text%%

----
Converted text to passive voice:
',
"1" => 'text'
    ),

"Convert this to active voice" => Array
    (
        "0" => 'Convert this text to active voice:

%%selected_text%%

----
Converted text to active voice:
',
"1" => 'text'
    ),

"Write a conclusion for this" => Array
    (
        "0" => 'Write a conclusion for this text:

%%selected_text%%

----
Conclusion:
',
"1" => 'text'
    ),

"Write a counterargument for this" => Array
    (
        "0" => 'Wite a counterargument for this text:

%%selected_text%%

----
Counterargument:
',
"1" => 'text'
    ),

"Translate this to Spanish" => Array
    (
"0" => 'Translate this text to Spanish:

%%selected_text%%

----
Spanish translation:
',
"1" => 'text'
    ),

"Generate an image idea for this" => Array
    (
"0" => 'Describe an image that would match this text:

%%selected_text%%

----
Image description:
',
"1" => 'text'
    ),

"Generate an image of this" => Array
    (
        "0" => 'A image of: %%selected_text%%',
"1" => 'image'
    )

);
    $rules  = get_option('aiomatic_assistant_list', array());
    if(!is_array($rules))
    {
        $rules = array();
    }
    if(empty($rules))
    {
        $rules = $aiomatic_assistant_defaults;
    }
    return $rules;
}
function aiomatic_hex2rgb($hex) {
    $hex = str_replace("#", "", $hex);
    if(strlen($hex) == 3) {
        $r = hexdec(substr($hex,0,1).substr($hex,0,1));
	    $g = hexdec(substr($hex,1,1).substr($hex,1,1));
	    $b = hexdec(substr($hex,2,1).substr($hex,2,1));
	} else {
	    $r = hexdec(substr($hex,0,2));
	    $g = hexdec(substr($hex,2,2));
	    $b = hexdec(substr($hex,4,2));
	}
	$rgb = array($r, $g, $b);
	return implode(",", $rgb);
}
function aiomatic_array_unique($array, $keep_key_assoc = false){
    $duplicate_keys = array();
    $tmp = array();       
 
    foreach ($array as $key => $val){
        if (is_object($val))
            $val = (array)$val;
 
        if (!in_array($val, $tmp))
            $tmp[] = $val;
        else
            $duplicate_keys[] = $key;
    }
 
    foreach ($duplicate_keys as $key)
        unset($array[$key]);
 
    return $keep_key_assoc ? $array : array_values($array);
 }
function aiomatic_clean_language_model_texts($content)
{
    $content = preg_replace('#As an? (?:AI )?languau?ge model(?: AI)?,?\s?#i' , '', $content);
    return $content;
}

function aiomatic_trailing_comma($incrementor, $count, &$subject) {
    $stopper = $count - 1;
	if ($incrementor !== $stopper) {
		return $subject .= ',';
	}
}
function aiomatic_auto_clear_log()
{
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
        include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
       wp_filesystem($creds);
    }
    if ($wp_filesystem->exists(WP_CONTENT_DIR . '/aiomatic_info.log')) {
        $wp_filesystem->delete(WP_CONTENT_DIR . '/aiomatic_info.log');
    }
}
function aiomatic_isSecure() {
    return
      (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off')
      || $_SERVER['SERVER_PORT'] == 443;
}

function aiomatic_get_mime ($filename) {
    $mime_types = array(
        'txt' => 'text/plain',
        'htm' => 'text/html',
        'html' => 'text/html',
        'php' => 'text/html',
        'css' => 'text/css',
        'js' => 'application/javascript',
        'json' => 'application/json',
        'xml' => 'application/xml',
        'swf' => 'application/x-shockwave-flash',
        'flv' => 'video/x-flv',
        'png' => 'image/png',
        'jpe' => 'image/jpeg',
        'jpeg' => 'image/jpeg',
        'jpg' => 'image/jpeg',
        'gif' => 'image/gif',
        'bmp' => 'image/bmp',
        'ico' => 'image/vnd.microsoft.icon',
        'tiff' => 'image/tiff',
        'mts' => 'video/mp2t',
        'tif' => 'image/tiff',
        'svg' => 'image/svg+xml',
        'svgz' => 'image/svg+xml',
        'zip' => 'application/zip',
        'rar' => 'application/x-rar-compressed',
        'exe' => 'application/x-msdownload',
        'msi' => 'application/x-msdownload',
        'cab' => 'application/vnd.ms-cab-compressed',
        'mp3' => 'audio/mpeg',
        'qt' => 'video/quicktime',
        'mov' => 'video/quicktime',
        'wmv' => 'video/x-ms-wmv',
        'mp4' => 'video/mp4',
        'm4p' => 'video/m4p',
        'm4v' => 'video/m4v',
        'mpg' => 'video/mpg',
        'mp2' => 'video/mp2',
        'mpe' => 'video/mpe',
        'mpv' => 'video/mpv',
        'm2v' => 'video/m2v',
        'm4v' => 'video/m4v',
        '3g2' => 'video/3g2',
        '3gpp' => 'video/3gpp',
        'f4v' => 'video/f4v',
        'f4p' => 'video/f4p',
        'f4a' => 'video/f4a',
        'f4b' => 'video/f4b',
        '3gp' => 'video/3gp',
        'avi' => 'video/x-msvideo',
        'mpeg' => 'video/mpeg',
        'mpegps' => 'video/mpeg',
        'webm' => 'video/webm',
        'mpeg4' => 'video/mp4',
        'mkv' => 'video/mkv',
        'pdf' => 'application/pdf',
        'psd' => 'image/vnd.adobe.photoshop',
        'ai' => 'application/postscript',
        'eps' => 'application/postscript',
        'ps' => 'application/postscript',
        'doc' => 'application/msword',
        'rtf' => 'application/rtf',
        'xls' => 'application/vnd.ms-excel',
        'ppt' => 'application/vnd.ms-powerpoint',
        'docx' => 'application/msword',
        'xlsx' => 'application/vnd.ms-excel',
        'pptx' => 'application/vnd.ms-powerpoint',
        'odt' => 'application/vnd.oasis.opendocument.text',
        'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
    );
    $ext = array_values(array_slice(explode('.', $filename), -1));$ext = $ext[0];

    if(stristr($filename, 'dailymotion.com'))
    {
        return 'application/octet-stream';
    }
    if (function_exists('mime_content_type')) {
        error_reporting(0);
        $mimetype = mime_content_type($filename);
        error_reporting(E_ALL);
        if($mimetype == '')
        {
            if (array_key_exists($ext, $mime_types)) {
                return $mime_types[$ext];
            } else {
                return 'application/octet-stream';
            }
        }
        return $mimetype;
    }
    elseif (function_exists('finfo_open')) {
        $finfo = finfo_open(FILEINFO_MIME);
        $mimetype = finfo_file($finfo, $filename);
        finfo_close($finfo);
        if($mimetype === false)
        {
            if (array_key_exists($ext, $mime_types)) {
                return $mime_types[$ext];
            } else {
                return 'application/octet-stream';
            }
        }
        return $mimetype;

    } elseif (array_key_exists($ext, $mime_types)) {
        return $mime_types[$ext];
    } else {
        return 'application/octet-stream';
    }
}
function aiomatict_get_items($query=array()){
     global $wpdb;
     $defaults = array(
       'post_hash'=>''
     );
 
    $query = wp_parse_args($query, $defaults);
    
    extract($query);
    $allowed_fields = aiomatict_get_item_table_columns();
    $select_sql = "SELECT post_result FROM {$wpdb->aiomatict_shortcode_rez}";
    $join_sql='';
    $where_sql = $wpdb->prepare("WHERE post_hash = %s", $post_hash);

    $sql = "$select_sql $where_sql";
    $logs = $wpdb->get_results($sql);
    $logs = apply_filters('aiomatict_get_items', $logs, $query);
    return $logs;
}
function replaceAIPostShortcodes($content, $post_link, $post_title, $blog_title, $post_excerpt, $post_content, $user_name, $featured_image, $post_cats, $post_tagz, $post_id, $img_attr = '', $old_title = '', $post_title_keywords = '', $custom_shortcodes = '', $global_prepend = '', $global_append = '')
{
    $matches = array();
    $i = 0;
    preg_match_all('~%regex\(\s*\"([^"]+?)\s*"\s*[,;]\s*\"([^"]*)\"\s*(?:[,;]\s*\"([^"]*?)\s*\")?(?:[,;]\s*\"([^"]*?)\s*\")?(?:[,;]\s*\"([^"]*?)\s*\")?\)%~si', $content, $matches);
    if (is_array($matches) && count($matches) && is_array($matches[0])) 
    {
        for($i = 0; $i < count($matches[0]); $i++)
        {
            if (isset($matches[0][$i])) $fullmatch = $matches[0][$i];
            if (isset($matches[1][$i])) $search_in = replaceAIPostShortcodes($matches[1][$i], $post_link, $post_title, $blog_title, $post_excerpt, $post_content, $user_name, $featured_image, $post_cats, $post_tagz, $post_id, $img_attr, $old_title, $post_title_keywords, $custom_shortcodes, $global_prepend, $global_append);
            if (isset($matches[2][$i])) $matchpattern = $matches[2][$i];
            if (isset($matches[3][$i])) $element = $matches[3][$i];
            if (isset($matches[4][$i])) $delimeter = $matches[4][$i];if (isset($matches[5][$i])) $counter = $matches[5][$i];
            if (isset($matchpattern)) {
               if (preg_match('<^[\/#%+~[\]{}][\s\S]*[\/#%+~[\]{}]$>', $matchpattern, $z)) {
                  $ret = preg_match_all($matchpattern, $search_in, $submatches, PREG_PATTERN_ORDER);
               }
               else {
                  $ret = preg_match_all('~'.$matchpattern.'~si', $search_in, $submatches, PREG_PATTERN_ORDER);
               }
            }
            if (isset($submatches)) {
               if (is_array($submatches)) {
                  $empty_elements = array_keys($submatches[0], "");
                  foreach ($empty_elements as $e) {
                     unset($submatches[0][$e]);
                  }
                  $submatches[0] = array_unique($submatches[0]);
                  if (!is_numeric($element)) {
                     $element = 0;
                  }if (!is_numeric($counter)) {
                     $counter = 0;
                  }
                  if(isset($submatches[(int)($element)]))
                  {
                      $matched = $submatches[(int)($element)];
                  }
                  else
                  {
                      $matched = '';
                  }
                  $matched = array_unique((array)$matched);
                  if (empty($delimeter) || $delimeter == 'null') {
                     if (isset($matched[$counter])) $matched = $matched[$counter];
                  }
                  else {
                     $matched = implode($delimeter, $matched);
                  }
                  if (empty($matched)) {
                     $content = str_replace($fullmatch, '', $content);
                  } else {
                     $content = str_replace($fullmatch, $matched, $content);
                  }
               }
            }
        }
    }
    $spintax = new AIomatic_Spintax();
    $content = $spintax->process($content);
    $pcxxx = explode('<!- template ->', $content);
    $content = $pcxxx[array_rand($pcxxx)];
    $content = str_replace('%%random_sentence%%', aiomatic_random_sentence_generator(), $content);
    $content = str_replace('%%random_sentence2%%', aiomatic_random_sentence_generator(false), $content);
    $content = aiomatic_replaceSynergyShortcodes($content);
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if (isset($aiomatic_Main_Settings['custom_html'])) {
        $content = str_replace('%%custom_html%%', $aiomatic_Main_Settings['custom_html'], $content);
    }
    if (isset($aiomatic_Main_Settings['custom_html2'])) {
        $content = str_replace('%%custom_html2%%', $aiomatic_Main_Settings['custom_html2'], $content);
    }
    $content = str_replace('%%post_link%%', $post_link, $content);
    $content = str_replace('%%post_title%%', $post_title, $content);
    $content = str_replace('%%post_title_keywords%%', $post_title_keywords, $content);
    $content = str_replace('%%post_original_title%%', $old_title, $content);
    $content = str_replace('%%blog_title%%', $blog_title, $content);
    $content = str_replace('%%post_excerpt%%', $post_excerpt, $content);
    $post_content = strip_shortcodes($post_content);
    $content = str_replace('%%post_content%%', $post_content, $content);
    $content = str_replace('%%post_content_plain_text%%', strip_tags($post_content), $content);
    $content = str_replace('%%author_name%%', $user_name, $content);
    $content = str_replace('%%current_date_time%%', date('Y/m/d H:i:s'), $content);
    $content = str_replace('%%featured_image%%', $featured_image, $content);
    $content = str_replace('%%post_cats%%', $post_cats, $content);
    $content = str_replace('%%post_tags%%', $post_tagz, $content);
    $img_attr = str_replace('%%image_source_name%%', '', $img_attr);
    $img_attr = str_replace('%%image_source_url%%', '', $img_attr);
    $img_attr = str_replace('%%image_source_website%%', '', $img_attr);
    $content = str_replace('%%royalty_free_image_attribution%%', $img_attr, $content);
    if($post_id != '')
    {
        preg_match_all('#%%!([^!]*?)!%%#', $content, $matched_content);
        if(isset($matched_content[1][0]))
        {
            foreach($matched_content[1] as $mc)
            {
                $post_custom_data = get_post_meta($post_id, $mc, true);
                if($post_custom_data != '')
                {
                    $content = str_replace('%%!' . $mc . '!%%', $post_custom_data, $content);
                }
                else
                {
                    $content = str_replace('%%!' . $mc . '!%%', '', $content);
                }
            }
        }
        preg_match_all('#%%!!([^!]*?)!!%%#', $content, $matched_content);
        if(isset($matched_content[1][0]))
        {
            foreach($matched_content[1] as $mc)
            {
                $ctaxs = '';
                $terms = get_the_terms( $post_id, $mc );
                if ( ! empty( $terms ) && ! is_wp_error( $terms ) )
                {
                    $ctaxs_arr = array();
                    foreach ( $terms as $term ) {
                        $ctaxs_arr[] = $term->slug;
                    }
                    $ctaxs = implode(',', $ctaxs_arr);
                }
                if($post_custom_data != '')
                {
                    $content = str_replace('%%!!' . $mc . '!!%%', $ctaxs, $content);
                }
                else
                {
                    $content = str_replace('%%!!' . $mc . '!!%%', '', $content);
                }
            }
        }
    }
    else
    {
        preg_match_all('#%%!([^!]*?)!%%#', $content, $matched_content);
        if(isset($matched_content[1][0]))
        {
            foreach($matched_content[1] as $mc)
            {
                $content = str_replace('%%!' . $mc . '!%%', '', $content);
            }
        }
        preg_match_all('#%%!!([^!]*?)!!%%#', $content, $matched_content);
        if(isset($matched_content[1][0]))
        {
            foreach($matched_content[1] as $mc)
            {
                $content = str_replace('%%!!' . $mc . '!!%%', '', $content);
            }
        }
    }
    if ( is_user_logged_in() ) 
    {
        $user_id = get_current_user_id();
        if($user_id !== 0)
        {
            preg_match_all('#%%~([^!]*?)~%%#', $content, $matched_content);
            if(isset($matched_content[1][0]))
            {
                foreach($matched_content[1] as $mc)
                {
                    $post_custom_data = get_user_meta($user_id, $mc, true);
                    if($post_custom_data != '')
                    {
                        $content = str_replace('%%~' . $mc . '~%%', $post_custom_data, $content);
                    }
                    else
                    {
                        $content = str_replace('%%~' . $mc . '~%%', '', $content);
                    }
                }
            }
        }
        else
        {
            preg_match_all('#%%~([^!]*?)~%%#', $content, $matched_content);
            if(isset($matched_content[1][0]))
            {
                foreach($matched_content[1] as $mc)
                {
                    $content = str_replace('%%~' . $mc . '~%%', '', $content);
                }
            }
        }
    } 
    else 
    {
        preg_match_all('#%%~([^!]*?)~%%#', $content, $matched_content);
        if(isset($matched_content[1][0]))
        {
            foreach($matched_content[1] as $mc)
            {
                $content = str_replace('%%~' . $mc . '~%%', '', $content);
            }
        }
    }
    $content = preg_replace_callback('#%%random_image_url\[([^\]]*?)\]%%#', function ($matches) {
        $arv = array();
        $my_img = aiomatic_get_random_image_google($matches[1], 0, 0, '', $arv);
        return $my_img;
    }, $content);
    $content = preg_replace_callback('#%%random_image\[([^\]]*?)\](\[\d+\])?%%#', function ($matches) {
        if(isset($matches[2]))
        {
            $chance = trim($matches[2], '[]');
        }
        else
        {
            $chance = '';
        }
        $arv = array();
        $my_img = aiomatic_get_random_image_google($matches[1], 0, 0, $chance, $arv);
        return '<img src="' . $my_img . '">';
    }, $content);
    $content = preg_replace_callback('#%%random_video\[([^\]]*?)\](\[\d+\])?%%#', function ($matches) {
        if(isset($matches[2]))
        {
            $chance = trim($matches[2], '[]');
        }
        else
        {
            $chance = '';
        }
        $my_vid = aiomoatic_get_video($matches[1], $chance);
        return $my_vid;
    }, $content);

    if (!isset($aiomatic_Main_Settings['app_id']) || trim($aiomatic_Main_Settings['app_id']) == '') 
    {
        aiomatic_log_to_file('You need to insert a valid OpenAI/AiomaticAPI API Key for the custom shortcode creator to work!');
    }
    else
    {
        $allmodels = aiomatic_get_all_models();
        $appids = preg_split('/\r\n|\r|\n/', trim($aiomatic_Main_Settings['app_id']));
        $appids = array_filter($appids);
        $token = $appids[array_rand($appids)];
        if(!is_array($custom_shortcodes))
        {
            if($custom_shortcodes != '')
            {
                $custom_shortcodes = replaceAIPostShortcodes($custom_shortcodes, $post_link, $post_title, $blog_title, $post_excerpt, $post_content, $user_name, $featured_image, $post_cats, $post_tagz, $post_id, $img_attr, $old_title, $post_title_keywords, '', $global_prepend, $global_append);
            }
            $custom_shortcodes = preg_split('/\r\n|\r|\n/', $custom_shortcodes);
        }
        foreach($custom_shortcodes as $my_short)
        {
            $name_part = explode('=>', $my_short);
            if(isset($name_part[1]) && !empty(trim($name_part[1])))
            {
                $shortname = trim($name_part[0]);
                if(strstr($content, '%%' . $shortname . '%%'))
                {
                    $shortval = '';
                    $ai_part = explode('@@', $name_part[1]);
                    if(isset($ai_part[1]) && !empty(trim($ai_part[1])))
                    {
                        if(!in_array(trim($ai_part[0]), $allmodels))
                        {
                            $aimodel = get_default_model_name($aiomatic_Main_Settings);
                        }
                        else
                        {
                            $aimodel = trim($ai_part[0]);
                        }
                        $ai_command = trim($ai_part[1]);
                        $ai_command = apply_filters('aiomatic_replace_aicontent_shortcode', $ai_command);
                        $max_tokens = aiomatic_get_max_tokens($aimodel);
                        $query_token_count = count(aiomatic_encode($ai_command));
                        $available_tokens = aiomatic_compute_available_tokens($aimodel, $max_tokens, $ai_command, $query_token_count);
                        if($available_tokens <= AIOMATIC_MINIMUM_TOKENS_FOR_COMPLETIONS)
                        {
                            $string_len = strlen($ai_command);
                            $string_len = $string_len / 2;
                            $string_len = intval(0 - $string_len);
                            $ai_command = aiomatic_substr($ai_command, 0, $string_len);
                            $ai_command = trim($ai_command);
                            $query_token_count = count(aiomatic_encode($ai_command));
                            $available_tokens = $max_tokens - $query_token_count;
                        }
                        if (isset($aiomatic_Main_Settings['enable_detailed_logging'])) 
                        {
                            $api_service = aiomatic_get_api_service($token);
                            aiomatic_log_to_file('Calling ' . $api_service . ' (' . $aimodel . ') for custom shortcode text: ' . $ai_command);
                        }
                        $thread_id = '';
                        $aierror = '';
                        $finish_reason = '';
                        $temperature = 1;
                        $top_p = 1;
                        $ai_command = aiomatic_replaceSynergyShortcodes($ai_command);
                        if(!empty($ai_command))
                        {
                            $ai_command = replaceAIPostShortcodes($ai_command, $post_link, $post_title, $blog_title, $post_excerpt, $post_content, $user_name, $featured_image, $post_cats, $post_tagz, $post_id, $img_attr, $old_title, $post_title_keywords, $custom_shortcodes, $global_prepend, $global_append);
                        }
                        $presence_penalty = 0;
                        $frequency_penalty = 0;
                        if(!empty($global_prepend))
                        {
                            $ai_command = $global_prepend . ' ' . $ai_command;
                        }
                        if(!empty($global_append))
                        {
                            $ai_command = $ai_command . ' ' . $global_append;
                        }
                        $generated_text = aiomatic_generate_text($token, $aimodel, $ai_command, $available_tokens, $temperature, $top_p, $presence_penalty, $frequency_penalty, false, 'customShortcode', 0, $finish_reason, $aierror, true, false, '', '', 'user', '', $thread_id, '', 'disabled');
                        if($generated_text === false)
                        {
                            aiomatic_log_to_file('Custom shortcode generator error: ' . $aierror);
                        }
                        else
                        {
                            $shortval = trim(trim(trim(trim($generated_text), '.'), ' "\''));
                        }
                    }
                    $content = str_replace('%%' . $shortname . '%%', $shortval, $content);
                }
            }
        }
    }
    $content = apply_filters('aiomatic_replace_aicontent_shortcode', $content);
    return $content;
}
function aiomatic_preg_grep_keys( $pattern, $input, $flags = 0 )
{
    if(!is_array($input))
    {
        return array();
    }
    $keys = preg_grep( $pattern, array_keys( $input ), $flags );
    $vals = array();
    foreach ( $keys as $key )
    {
        $vals[$key] = $input[$key];
    }
    return $vals;
}
function aiomatic_select_ai_image($new_post_title, $image_url)
{
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if (isset($aiomatic_Main_Settings['use_image_ai']) && $aiomatic_Main_Settings['use_image_ai'] === 'on') 
    {
        if (!isset($aiomatic_Main_Settings['app_id']) || trim($aiomatic_Main_Settings['app_id']) == '') 
        {
            aiomatic_log_to_file('You need to insert a valid OpenAI/AiomaticAPI API Key for this to work!');
        }
        else
        {
            if (isset($aiomatic_Main_Settings['image_ai_prompt']) && trim($aiomatic_Main_Settings['image_ai_prompt']) != '') 
            {
                $prompt = trim($aiomatic_Main_Settings['image_ai_prompt']);
            }
            else
            {
                $prompt = 'Select an image URL, based on its file name, which matches the best the a post, based on its title. If no matching image can be selected, pick a random one from the list. Respond only with the URL of the selected image and with nothing else. The title of the post is: \"%%post_title%%\" The image URL list is: %%image_list%%';
            }
            if (isset($aiomatic_Main_Settings['image_ai_model']) && trim($aiomatic_Main_Settings['image_ai_model']) != '') 
            {
                $model = trim($aiomatic_Main_Settings['image_ai_model']);
            }
            else
            {
                $model = get_default_model_name($aiomatic_Main_Settings);
            }
            if (isset($aiomatic_Main_Settings['img_assistant_id']) && trim($aiomatic_Main_Settings['img_assistant_id']) != '') 
            {
                $img_assistant_id = trim($aiomatic_Main_Settings['img_assistant_id']);
            }
            else
            {
                $img_assistant_id = '';
            }
            $all_models = aiomatic_get_all_models(true);
            if(!in_array($model, $all_models))
            {
                $model = get_default_model_name($aiomatic_Main_Settings);
            }
            $appids = preg_split('/\r\n|\r|\n/', trim($aiomatic_Main_Settings['app_id']));
            $appids = array_filter($appids);
            $token = $appids[array_rand($appids)];
            $prompt = str_replace('%%post_title%%', $new_post_title, $prompt);
            $image_url_arr = explode(',', $image_url);
            $changemade = false;
            foreach($image_url_arr as $this_ind => $img_url_me)
            {
                $img_url_me = trim($img_url_me);
                if(is_numeric($img_url_me))
                {
                    $attachment_url = wp_get_attachment_url($img_url_me);
                    if($attachment_url !== false)
                    {
                        $image_url_arr[$this_ind] = $attachment_url . '(' . $img_url_me . ')';
                        $changemade = true;
                    }
                }
            }
            if($changemade == true)
            {
                $image_url = implode(',', $image_url_arr);
            }
            $prompt = str_replace('%%image_list%%', $image_url, $prompt);
            $query_token_count = count(aiomatic_encode($prompt));
            $max_tokens = aiomatic_get_max_tokens($model);
            $available_tokens = aiomatic_compute_available_tokens($model, $max_tokens, $prompt, $query_token_count);
            if($available_tokens <= AIOMATIC_MINIMUM_TOKENS_FOR_COMPLETIONS)
            {
                $string_len = strlen($prompt);
                $string_len = $string_len / 2;
                $string_len = intval(0 - $string_len);
                $prompt = aiomatic_substr($prompt, 0, $string_len);
                $prompt = trim($prompt);
                $query_token_count = count(aiomatic_encode($prompt));
                $available_tokens = $max_tokens - $query_token_count;
            }
            if(empty($prompt))
            {
                aiomatic_log_to_file('Incorrect AI Image Selector prompt provided: ' . print_r($prompt, true));
            }
            else
            {
                $thread_id = '';
                $aierror = '';
                $finish_reason = '';
                $generated_text = aiomatic_generate_text($token, $model, $prompt, $available_tokens, 1, 1, 0, 0, false, 'AIImageSelector', 0, $finish_reason, $aierror, false, false, '', '', 'user', $img_assistant_id, $thread_id, '', 'disabled');
                if($generated_text === false)
                {
                    if (isset($aiomatic_Main_Settings['enable_detailed_logging'])) {
                        aiomatic_log_to_file('Failed to select the AI image for: ' . print_r($prompt, true) . ' - error: ' . $aierror);
                    }
                }
                else
                {
                    $selected_image = aiomatic_sanitize_ai_result($generated_text);
                    if (isset($aiomatic_Main_Settings['enable_detailed_logging'])) {
                        aiomatic_log_to_file('AI image selector result: ' . print_r($selected_image, true));
                    }
                    if(strstr($image_url, $selected_image) !== false)
                    {
                        preg_match_all('#^https?:\/\/.*\((\d+)\)$#i', $selected_image, $imatches);
                        if(isset($imatches[1][0]))
                        {
                            $selected_image = $imatches[1][0];
                        }
                        return $selected_image;
                    }
                    else
                    {
                        if (isset($aiomatic_Main_Settings['enable_detailed_logging'])) {
                            aiomatic_log_to_file('Returned content not found in input URL list: ' . print_r($selected_image, true));
                        }
                    }
                }
            }
        }
    }
    return false;
}
function aiomoatic_get_video($new_post_title, $chance = '')
{
    if($chance != '' && is_numeric($chance))
    {
        $chance = intval($chance);
        if(mt_rand(0, 99) >= $chance)
        {
            return '';
        }
    }
    $retme = '';
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if (isset($aiomatic_Main_Settings['improve_yt_kw']) && $aiomatic_Main_Settings['improve_yt_kw'] === 'on') 
    {
        if (!isset($aiomatic_Main_Settings['app_id']) || trim($aiomatic_Main_Settings['app_id']) == '') 
        {
            aiomatic_log_to_file('You need to insert a valid OpenAI/AiomaticAPI API Key for this to work!');
        }
        else
        {
            if (isset($aiomatic_Main_Settings['yt_kw_prompt']) && trim($aiomatic_Main_Settings['yt_kw_prompt']) != '') 
            {
                $prompt = trim($aiomatic_Main_Settings['yt_kw_prompt']);
            }
            else
            {
                $prompt = 'Using which keyword or search phrase should I search YouTube, to get the most relevant videos for this text: "%%aiomatic_query%%"';
            }
            if (isset($aiomatic_Main_Settings['yt_kw_model']) && trim($aiomatic_Main_Settings['yt_kw_model']) != '') 
            {
                $model = trim($aiomatic_Main_Settings['yt_kw_model']);
            }
            else
            {
                $model = get_default_model_name($aiomatic_Main_Settings);
            }
            if (isset($aiomatic_Main_Settings['yt_assistant_id']) && trim($aiomatic_Main_Settings['yt_assistant_id']) != '') 
            {
                $yt_assistant_id = trim($aiomatic_Main_Settings['yt_assistant_id']);
            }
            else
            {
                $yt_assistant_id = '';
            }
            $all_models = aiomatic_get_all_models(true);
            if(!in_array($model, $all_models))
            {
                $model = get_default_model_name($aiomatic_Main_Settings);
            }
            $appids = preg_split('/\r\n|\r|\n/', trim($aiomatic_Main_Settings['app_id']));
            $appids = array_filter($appids);
            $token = $appids[array_rand($appids)];
            $prompt = str_replace('%%query%%', $new_post_title, $prompt);
            $prompt = str_replace('%%aiomatic_query%%', $new_post_title, $prompt);
            $query_token_count = count(aiomatic_encode($prompt));
            $max_tokens = aiomatic_get_max_tokens($model);
            $available_tokens = aiomatic_compute_available_tokens($model, $max_tokens, $prompt, $query_token_count);
            if($available_tokens <= AIOMATIC_MINIMUM_TOKENS_FOR_COMPLETIONS)
            {
                $string_len = strlen($prompt);
                $string_len = $string_len / 2;
                $string_len = intval(0 - $string_len);
                $prompt = aiomatic_substr($prompt, 0, $string_len);
                $prompt = trim($prompt);
                $query_token_count = count(aiomatic_encode($prompt));
                $available_tokens = $max_tokens - $query_token_count;
            }
            if(empty($prompt))
            {
                aiomatic_log_to_file('Incorrect YouTube search keyword extractor prompt provided: ' . print_r($prompt, true));
            }
            else
            {
                $thread_id = '';
                $aierror = '';
                $finish_reason = '';
                $generated_text = aiomatic_generate_text($token, $model, $prompt, $available_tokens, 1, 1, 0, 0, false, 'YouTubeKeywordWriter', 0, $finish_reason, $aierror, false, false, '', '', 'user', $yt_assistant_id, $thread_id, '', 'disabled');
                if($generated_text === false)
                {
                    aiomatic_log_to_file('Failed to extract YouTube search keywords for: ' . print_r($prompt, true) . ' - error: ' . $aierror);
                }
                else
                {
                    $new_post_title = aiomatic_sanitize_ai_result($generated_text);
                }
            }
        }
    }
    if (isset($aiomatic_Main_Settings['yt_app_id']) && trim($aiomatic_Main_Settings['yt_app_id']) != '') {
        $items = array();
        $za_app = explode(',', $aiomatic_Main_Settings['yt_app_id']);
        $za_app = trim($za_app[array_rand($za_app)]);
        $feed_uri = 'https://www.googleapis.com/youtube/v3/search?part=snippet&type=video&key=' . $za_app;
        $feed_uri .= '&maxResults=10';
        $feed_uri .= '&q='.urlencode(trim(stripslashes(str_replace('&quot;', '"', $new_post_title))));
        $ch  = curl_init();
        if ($ch !== FALSE) 
        {
            if (isset($aiomatic_Main_Settings['proxy_url']) && $aiomatic_Main_Settings['proxy_url'] != '') {
                $prx = explode(',', $aiomatic_Main_Settings['proxy_url']);
                $randomness = array_rand($prx);
                curl_setopt( $ch, CURLOPT_PROXY, trim($prx[$randomness]));
                if (isset($aiomatic_Main_Settings['proxy_auth']) && $aiomatic_Main_Settings['proxy_auth'] != '') 
                {
                    $prx_auth = explode(',', $aiomatic_Main_Settings['proxy_auth']);
                    if(isset($prx_auth[$randomness]) && trim($prx_auth[$randomness]) != '')
                    {
                        curl_setopt( $ch, CURLOPT_PROXYUSERPWD, trim($prx_auth[$randomness]));
                    }
                }
            }
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            if (isset($aiomatic_Main_Settings['max_timeout']) && $aiomatic_Main_Settings['max_timeout'] != '')
            {
                $ztime = intval($aiomatic_Main_Settings['max_timeout']);
            }
            else
            {
                $ztime = 300;
            }
            curl_setopt($ch, CURLOPT_TIMEOUT, $ztime);
            curl_setopt($ch, CURLOPT_HTTPGET, 1);
            curl_setopt($ch, CURLOPT_REFERER, get_site_url());
            curl_setopt($ch, CURLOPT_URL, $feed_uri);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            $exec = curl_exec($ch);
            curl_close($ch);
            if ($exec !== FALSE) {
                $json  = json_decode($exec);
                if(isset($json->items))
                {
                    $items = $json->items;
                    if (count($items) == 0) 
                    {
                        $feed_uri = 'https://www.googleapis.com/youtube/v3/search?part=snippet&type=video&key=' . $za_app;
                        $feed_uri .= '&maxResults=10';
                        $keyword_class = new Aiomatic_keywords();
                        $new_post_title = $keyword_class->keywords($new_post_title, 1);
                        $feed_uri .= '&q='.urlencode(trim(stripslashes(str_replace('&quot;', '"', $new_post_title))));
                        $ch  = curl_init();
                        if ($ch !== FALSE) 
                        {
                            if (isset($aiomatic_Main_Settings['proxy_url']) && $aiomatic_Main_Settings['proxy_url'] != '') {
                                $prx = explode(',', $aiomatic_Main_Settings['proxy_url']);
                                $randomness = array_rand($prx);
                                curl_setopt( $ch, CURLOPT_PROXY, trim($prx[$randomness]));
                                if (isset($aiomatic_Main_Settings['proxy_auth']) && $aiomatic_Main_Settings['proxy_auth'] != '') 
                                {
                                    $prx_auth = explode(',', $aiomatic_Main_Settings['proxy_auth']);
                                    if(isset($prx_auth[$randomness]) && trim($prx_auth[$randomness]) != '')
                                    {
                                        curl_setopt( $ch, CURLOPT_PROXYUSERPWD, trim($prx_auth[$randomness]));
                                    }
                                }
                            }
                            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
                            if (isset($aiomatic_Main_Settings['max_timeout']) && $aiomatic_Main_Settings['max_timeout'] != '')
                            {
                                $ztime = intval($aiomatic_Main_Settings['max_timeout']);
                            }
                            else
                            {
                                $ztime = 300;
                            }
                            curl_setopt($ch, CURLOPT_TIMEOUT, $ztime);
                            curl_setopt($ch, CURLOPT_HTTPGET, 1);
                            curl_setopt($ch, CURLOPT_REFERER, get_site_url());
                            curl_setopt($ch, CURLOPT_URL, $feed_uri);
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
                            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
                            $exec = curl_exec($ch);
                            curl_close($ch);
                            if ($exec === FALSE) {
                                $json  = json_decode($exec);
                                if(isset($json->items))
                                {
                                    $items = $json->items;
                                }
                            }
                        }
                        else
                        {
                            aiomatic_log_to_file('Failed to init curl in YouTube API listing x2: ' . $feed_uri);
                        }
                    }
                }
                else
                {
                    if (isset($aiomatic_Main_Settings['enable_detailed_logging'])) {
                        aiomatic_log_to_file('YouTube API returned error: ' . $exec);
                    }
                }
            }
        }
        else
        {
            aiomatic_log_to_file('Failed to init curl in YouTube API listing: ' . $feed_uri);
        }
        if(isset($items[0]->id->videoId))
        {
            $rand_ind = array_rand($items);
            $video_id = $items[$rand_ind]->id->videoId;
            if (isset($aiomatic_Main_Settings['player_width']) && $aiomatic_Main_Settings['player_width'] !== '') {
                $width = esc_attr($aiomatic_Main_Settings['player_width']);
            }
            else
            {
                $width = 580;
            }
            if (isset($aiomatic_Main_Settings['player_height']) && $aiomatic_Main_Settings['player_height'] !== '') {
                $height = esc_attr($aiomatic_Main_Settings['player_height']);
            }
            else
            {
                $height = 380;
            }
            $retme = '<br/><br/><div class="automaticx-video-container"><iframe allow="autoplay" width="' . $width . '" height="' . $height . '" src="https://www.youtube.com/embed/' . $video_id . '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe></div>';
        }
    }
    else
    {
        $retme = aiomatic_get_youtube_video(trim(stripslashes(str_replace('&quot;', '"', $new_post_title))), $chance);
    }
    return $retme;
}
function aiomatic_get_youtube_video($keyword, $chance = '')
{
    if($chance != '' && is_numeric($chance))
    {
        $chance = intval($chance);
        if(mt_rand(0, 99) >= $chance)
        {
            return '';
        }
    }
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if (isset($aiomatic_Main_Settings['player_width']) && $aiomatic_Main_Settings['player_width'] !== '') {
        $width = esc_attr($aiomatic_Main_Settings['player_width']);
    }
    else
    {
        $width = 580;
    }
    if (isset($aiomatic_Main_Settings['player_height']) && $aiomatic_Main_Settings['player_height'] !== '') {
        $height = esc_attr($aiomatic_Main_Settings['player_height']);
    }
    else
    {
        $height = 380;
    }
    $res = aiomatic_file_get_contents_advanced('https://www.youtube.com/results?search_query=' . urlencode($keyword), '', 'self', 'Mozilla/5.0 (Windows NT 10.0;WOW64;rv:97.0) Gecko/20100101 Firefox/97.0/3871tuT2p1u-81');
    preg_match_all('/"\/watch\?v=([^"&?\/\s]{11})"/', $res, $matches);
    if(isset($matches[1]))
    {
        $items = $matches[1];
        if (count($items) > 0) 
        {
            return '<br/><br/><div class="automaticx-video-container"><iframe allow="autoplay" width="' . $width . '" height="' . $height . '" src="https://www.youtube.com/embed/' . $items[rand(0, count($items) - 1)] . '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe></div>';
        }
    }
    return '';
}
function aiomatic_generate_thumbmail( $post_id )
{    
    $post = get_post($post_id);
    $post_parent_id = $post->post_parent === 0 ? $post->ID : $post->post_parent;
    if ( has_post_thumbnail($post_parent_id) )
    {
        if ($id_attachment = get_post_thumbnail_id($post_parent_id)) {
            $the_image  = wp_get_attachment_url($id_attachment, false);
            return $the_image;
        }
    }
    $attachments = array_values(get_children(array(
        'post_parent' => $post_parent_id, 
        'post_status' => 'inherit', 
        'post_type' => 'attachment', 
        'post_mime_type' => 'image', 
        'order' => 'ASC', 
        'orderby' => 'menu_order ID') 
    ));
    if( sizeof($attachments) > 0 ) {
        $the_image  = wp_get_attachment_url($attachments[0]->ID, false);
        return $the_image;
    }
    $image_url = aiomatic_extractThumbnail($post->post_content);
    return $image_url;
}
function aiomatic_extractThumbnail($content) {
    $att = aiomatic_getUrls($content);
    if(count($att) > 0)
    {
        foreach($att as $link)
        {
            $mime = aiomatic_get_mime($link);
            if(stristr($mime, "image/") !== FALSE){
                return $link;
            }
        }
    }
    else
    {
        return '';
    }
    return '';
}
function aiomatic_getUrls($string) {
    $regex = '/https?\:\/\/[^\"\' \n\s]+/i';
    preg_match_all($regex, $string, $matches);
    return ($matches[0]);
}

function aiomatic_strip_html_tags($str)
{
    $str = html_entity_decode($str);
    $str1 = preg_replace('/(<|>)\1{2}/is', '', $str);
    if($str1 !== null)
    {
        $str = $str1;
    }
    $str1 = preg_replace(array(
        '@<head[^>]*?>.*?</head>@siu',
        '@<style[^>]*?>.*?</style>@siu',
        '@<script[^>]*?.*?</script>@siu',
        '@<noscript[^>]*?.*?</noscript>@siu'
    ), "", $str);
    if($str1 !== null)
    {
        $str = $str1;
    }
    $str = str_replace('><', '> <', $str);
    $str = strip_tags($str);
    return $str;
}
function aiomatic_get_base64_from_url($image_url) 
{
    $image_content = aiomatic_get_web_page($image_url);
    if ($image_content == false) {
        return false;
    }
    $base64_image = base64_encode($image_content);
    return $base64_image;
}
function aiomatic_base64_to_jpeg($base64_string, $output_file, $ret_path) 
{
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
        include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
       wp_filesystem($creds);
    }
    if ($wp_filesystem->exists($output_file)) 
    {
        return array($output_file, $ret_path); 
    }
    $ifp = fopen($output_file, 'wb'); 
    if($ifp !== false)
    {
        $decoded = base64_decode($base64_string);
        if($ifp !== false)
        {
            $decoded_temp = aiomatic_string_to_string_compress($decoded);
            if($decoded_temp !== false)
            {
                $decoded = $decoded_temp;
            }
            $rez = fwrite($ifp, $decoded);
            if($rez === false)
            {
                aiomatic_log_to_file('Failed to write file: ' . $output_file);
                return false;
            }
        }
        else
        {
            aiomatic_log_to_file('Failed to decode response file: ' . $base64_string);
            return false;
        }
        fclose($ifp);
    }
    else
    {
        aiomatic_log_to_file('Failed to open file: ' . $output_file);
        return false;
    }
    return array($output_file, $ret_path); 
}
function aiomatic_base64_to_file($base64_string, $output_file, $ret_path) 
{
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
        include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
       wp_filesystem($creds);
    }
    if ($wp_filesystem->exists($output_file)) 
    {
        return array($output_file, $ret_path); 
    }
    $ifp = fopen($output_file, 'wb'); 
    if($ifp !== false)
    {
        $decoded = base64_decode($base64_string);
        if($ifp !== false)
        {
            $rez = fwrite($ifp, $decoded);
            if($rez === false)
            {
                aiomatic_log_to_file('Failed to write file: ' . $output_file);
                return false;
            }
        }
        else
        {
            aiomatic_log_to_file('Failed to decode response file: ' . $base64_string);
            return false;
        }
        fclose($ifp);
    }
    else
    {
        aiomatic_log_to_file('Failed to open file: ' . $output_file);
        return false;
    }
    return array($output_file, $ret_path); 
}
function aiomatic_random_sentence_generator($first = true)
{
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if ($first == false) {
        $r_sentences = $aiomatic_Main_Settings['sentence_list2'];
    } else {
        $r_sentences = $aiomatic_Main_Settings['sentence_list'];
    }
    $r_variables = $aiomatic_Main_Settings['variable_list'];
    $r_sentences = trim($r_sentences);
    $r_variables = trim($r_variables, ';');
    $r_variables = trim($r_variables);
    $r_sentences = str_replace("\r\n", "\n", $r_sentences);
    $r_sentences = str_replace("\r", "\n", $r_sentences);
    $r_sentences = explode("\n", $r_sentences);
    $r_variables = str_replace("\r\n", "\n", $r_variables);
    $r_variables = str_replace("\r", "\n", $r_variables);
    $r_variables = explode("\n", $r_variables);
    $r_vars      = array();
    for ($x = 0; $x < count($r_variables); $x++) {
        $var = explode("=>", trim($r_variables[$x]));
        if (isset($var[1])) {
            $key          = strtolower(trim($var[0]));
            $words        = explode(";", trim($var[1]));
            $r_vars[$key] = $words;
        }
    }
    $max_s    = count($r_sentences) - 1;
    $rand_s   = rand(0, $max_s);
    $sentence = $r_sentences[$rand_s];
    $sentence = str_replace(' ,', ',', ucfirst(aiomatic_replace_words($sentence, $r_vars)));
    $sentence = str_replace(' .', '.', $sentence);
    $sentence = str_replace(' !', '!', $sentence);
    $sentence = str_replace(' ?', '?', $sentence);
    $sentence = trim($sentence);
    return $sentence;
}
function aiomatic_get_random_user_agent() {
	$agents = array(
		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36",
		"Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/603.3.8 (KHTML, like Gecko) Version/10.1.2 Safari/603.3.8",
		"Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36",
		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36",
		"Mozilla/5.0 (Windows NT 10.0; WOW64; rv:55.0) Gecko/20100101 Firefox/55.0",
		"Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:55.0) Gecko/20100101 Firefox/55.0",
		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36",
		"Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko",
		"Mozilla/5.0 (Windows NT 6.1; WOW64; rv:55.0) Gecko/20100101 Firefox/55.0",
		"Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:55.0) Gecko/20100101 Firefox/55.0",
		"Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.116 Safari/537.36 Edge/15.15063",
		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.12; rv:55.0) Gecko/20100101 Firefox/55.0",
		"Mozilla/5.0 (Windows NT 10.0; WOW64; rv:54.0) Gecko/20100101 Firefox/54.0",
		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
		"Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36"
	);
	$rand   = rand( 0, count( $agents ) - 1 );
	return trim( $agents[ $rand ] );
}
function aiomatic_admin_footer()
{
?>
    <div class="aiomatic-overlay" style="display: none">
        <div class="aiomatic_modal">
            <div class="aiomatic_modal_head">
                <span class="aiomatic_modal_title"><?php echo esc_html__('Aiomatic', 'aiomatic-automatic-ai-content-writer');?></span>
                <span class="aiomatic_modal_close">&times;</span>
            </div>
            <div class="aiomatic_modal_content"></div>
        </div>
    </div>
    <div class="wpcgai_lds-ellipsis" style="display: none">
        <div class="aiomatic-generating-title"><?php echo esc_html__('Loading content...', 'aiomatic-automatic-ai-content-writer');?></div>
        <div class="aiomatic-generating-process"></div>
        <div class="aiomatic-timer"></div>
    </div>
<?php
}
function aiomatic_assign_var(&$target, $var, $root = false) {
	static $cnt = 0;
    $key = key($var);
    if(is_array($var[$key])) 
        aiomatic_assign_var($target[$key], $var[$key], false);
    else {
        if($key==0)
		{
			if($cnt == 0 && $root == true)
			{
				$target['_aiomaticr_nonce'] = $var[$key];
				$cnt++;
			}
			elseif($cnt == 1 && $root == true)
			{
				$target['_wp_http_referer'] = $var[$key];
				$cnt++;
			}
			else
			{
				$target[] = $var[$key];
			}
		}
        else
		{
            $target[$key] = $var[$key];
		}
    }   
}
function aiomatic_utf8ize($arr){
    if (is_array($arr)) {
        foreach ($arr as $k => $v) {
            $arr[$k] = aiomatic_utf8ize($v);
        }
    } else if (is_string ($arr)) {
        return aiomatic_utf8_encode($arr);
    }
    return $arr;
}
function aiomatic_safe_json_encode($value){
    $encoded = json_encode($value);
    switch (json_last_error()) {
        case JSON_ERROR_NONE:
            return $encoded;
        case JSON_ERROR_DEPTH:
            throw new Exception('Maximum stack depth exceeded');
        case JSON_ERROR_STATE_MISMATCH:
            throw new Exception('Underflow or the modes mismatch');
        case JSON_ERROR_CTRL_CHAR:
            throw new Exception('Unexpected control character found');
        case JSON_ERROR_SYNTAX:
            throw new Exception('Syntax error, malformed JSON');
        case JSON_ERROR_UTF8:
            $clean = aiomatic_utf8ize($value);
            return aiomatic_safe_json_encode($clean);
        default:
            throw new Exception('Unknown error in json encoding');
    }
}
function aiomatic_split_to_token_len($tokens, $max_len)
{
    $ret_me = array();
    if(count($tokens) > $max_len)
    {
        $chunks = array_chunk($tokens, $max_len, true);
        foreach($chunks as $thisch)
        {
            $ret_me[] = aiomatic_decode($thisch);
        }
    }
    else
    {
        $ret_me[] = aiomatic_decode($tokens);
    }
    return $ret_me;
}
use Gioni06\Gpt3Tokenizer\Gpt3TokenizerConfig;
use Gioni06\Gpt3Tokenizer\Gpt3Tokenizer;
function aiomatic_decode($tokens) 
{
    if (version_compare(PHP_VERSION, '8.0.2', '>=') && extension_loaded('mbstring')) 
    {
        require_once (dirname(__FILE__) . "/res/tokenizer/Gpt3TokenizerConfig.php"); 
        require_once (dirname(__FILE__) . "/res/tokenizer/Merges.php"); 
        require_once (dirname(__FILE__) . "/res/tokenizer/Vocab.php"); 
        require_once (dirname(__FILE__) . "/res/tokenizer/Gpt3Tokenizer.php"); 
        $config = new Gpt3TokenizerConfig();
        $tokenizer = new Gpt3Tokenizer($config);
        $text = $tokenizer->decode($tokens);
        return $text;
    }
    else
    {
        return aiomatic_decode_old($tokens);
    }
}
function aiomatic_decode_old($tokens) 
{
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') )
    {
        include_once(ABSPATH . 'wp-admin/includes/file.php');
        $creds = request_filesystem_credentials( site_url() );
        wp_filesystem($creds);
    }
    $rencoder = $wp_filesystem->get_contents(dirname(__FILE__) . "/res/encoder.json");
    $encoder = json_decode($rencoder, true);
    if(empty($encoder))
    {
        aiomatic_log_to_file('Failed to load encoder.json: ' . $rencoder);
        return false;
    }
    $decoder = array();
    foreach($encoder as $index => $val)
    {
        $decoder[$val] = $index;
    }
    $raw_chars = $wp_filesystem->get_contents(dirname(__FILE__) . "/res/characters.json");
    $byte_encoder = json_decode($raw_chars, true);
    if(empty($byte_encoder))
    {
        aiomatic_log_to_file('Failed to load characters.json: ' . $raw_chars);
        return false;
    }
    $byte_decoder = array();
    foreach($byte_encoder as $index => $val)
    {
        $byte_decoder[$val] = $index;
    }
    $text = '';
    $mych_arr = [];
    foreach($tokens as $myt)
    {
        if(isset($decoder[$myt]))
        {
            $mych_arr[] = $decoder[$myt];
        }
        else
        {
            aiomatic_log_to_file('Character not found in decoder: ' . $myt);
        }
    }
    $text = implode('', $mych_arr);
    $text_arr = preg_split('//u', $text, -1, PREG_SPLIT_NO_EMPTY);
    $final_arr = array();
    foreach($text_arr as $txa)
    {
        if(isset($byte_decoder[$txa]))
        {
            $final_arr[] = $byte_decoder[$txa];
        }
        else
        {
            aiomatic_log_to_file('Character not found in byte_decoder: ' . $txa);
        }
    }
    $output = '';
    for ($i = 0, $j = count($final_arr); $i < $j; ++$i) {
        $output .= chr($final_arr[$i]);
    }
    return $output;
}
function aiomatic_encode($text) 
{
    if (version_compare(PHP_VERSION, '8.0.2', '>=') && extension_loaded('mbstring')) 
    {
        require_once (dirname(__FILE__) . "/res/tokenizer/Gpt3TokenizerConfig.php"); 
        require_once (dirname(__FILE__) . "/res/tokenizer/Merges.php"); 
        require_once (dirname(__FILE__) . "/res/tokenizer/Vocab.php"); 
        require_once (dirname(__FILE__) . "/res/tokenizer/Gpt3Tokenizer.php"); 
        $config = new Gpt3TokenizerConfig();
        $tokenizer = new Gpt3Tokenizer($config);
        $tokens = $tokenizer->encode($text);
        return $tokens;
    }
    else
    {
        return aiomatic_encode_old($text);
    }
}
function aiomatic_encode_old($text) 
{
    $bpe_tokens = array();
    if(empty($text))
    {
        return $bpe_tokens;
    }
    global $wp_filesystem;
    if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') )
    {
        include_once(ABSPATH . 'wp-admin/includes/file.php');
        $creds = request_filesystem_credentials( site_url() );
        wp_filesystem($creds);
    }
    $raw_chars = $wp_filesystem->get_contents(dirname(__FILE__) . "/res/characters.json");
    $byte_encoder = json_decode($raw_chars, true);
    if(empty($byte_encoder))
    {
        aiomatic_log_to_file('Failed to load characters.json: ' . $raw_chars);
        return $bpe_tokens;
    }
    $rencoder = $wp_filesystem->get_contents(dirname(__FILE__) . "/res/encoder.json");
    $encoder = json_decode($rencoder, true);
    if(empty($encoder))
    {
        aiomatic_log_to_file('Failed to load encoder.json: ' . $rencoder);
        return $bpe_tokens;
    }

    $bpe_file = $wp_filesystem->get_contents(dirname(__FILE__) . "/res/vocab.bpe");
    if(empty($bpe_file))
    {
        aiomatic_log_to_file('Failed to load vocab.bpe');
        return $bpe_tokens;
    }
    $text = str_replace ("\r\n", "\n", $text);
    preg_match_all("#'s|'t|'re|'ve|'m|'ll|'d| ?\p{L}+| ?\p{N}+| ?[^\s\p{L}\p{N}]+|\s+(?!\S)|\s+#u", $text, $matches);
    if(!isset($matches[0]) || count($matches[0]) == 0)
    {
        aiomatic_log_to_file('Failed to match string: ' . $text);
        return $bpe_tokens;
    }
    $lines = preg_split('/\r\n|\r|\n/', $bpe_file);
    $bpe_merges = array();
    $bpe_merges_temp = array_slice($lines, 1, count($lines), true);
    foreach($bpe_merges_temp as $bmt)
    {
        $split_bmt = preg_split('#(\s+)#', $bmt);
        $split_bmt = array_filter($split_bmt, 'aiomatic_myFilter');
        if(count($split_bmt) > 0)
        {
            $bpe_merges[] = $split_bmt;
        }
    }
    $bpe_ranks = aiomatic_dictZip($bpe_merges, range(0, count($bpe_merges) - 1));
    
    $cache = array();
    foreach($matches[0] as $token)
    {
        $new_tokens = array();
        $chars = array();
        $token = aiomatic_utf8_encode($token);
        if(function_exists('mb_strlen'))
        {
            $len = mb_strlen($token, 'UTF-8');
            for ($i = 0; $i < $len; $i++) {
                $chars[] = mb_substr($token, $i, 1, 'UTF-8');
            }
        }
        else
        {
            $chars = str_split($token);
        }
        $result_word = '';
        foreach($chars as $char)
        {
            if(isset($byte_encoder[aiomatic_unichr($char)]))
            {
                $result_word .= $byte_encoder[aiomatic_unichr($char)];
            }
        }
        $new_tokens_bpe = aiomatic_bpe($result_word, $bpe_ranks, $cache);
        $new_tokens_bpe = explode(' ', $new_tokens_bpe);
        foreach($new_tokens_bpe as $x)
        {
            if(isset($encoder[$x]))
            {
                if(isset($new_tokens[$x]))
                {
                    $new_tokens[rand() . '---' . $x] = $encoder[$x];
                }
                else
                {
                    $new_tokens[$x] = $encoder[$x];
                }
            }
            else
            {
                if(isset($new_tokens[$x]))
                {
                    $new_tokens[rand() . '---' . $x] = $x;
                }
                else
                {
                    $new_tokens[$x] = $x;
                }
            }
        }
        foreach($new_tokens as $ninx => $nval)
        {
            if(isset($bpe_tokens[$ninx]))
            {
                $bpe_tokens[rand() . '---' . $ninx] = $nval;
            }
            else
            {
                $bpe_tokens[$ninx] = $nval;
            }
        }
    }
    return $bpe_tokens;
}
function aiomatic_myFilter($var)
{
    return ($var !== NULL && $var !== FALSE && $var !== '');
}

function aiomatic_unichr($c) 
{
    if (ord($c[0]) >=0 && ord($c[0]) <= 127)
    {
        return ord($c[0]);
    }
    if (ord($c[0]) >= 192 && ord($c[0]) <= 223)
    {
        return (ord($c[0])-192)*64 + (ord($c[1])-128);
    }
    if (ord($c[0]) >= 224 && ord($c[0]) <= 239)
    {
        return (ord($c[0])-224)*4096 + (ord($c[1])-128)*64 + (ord($c[2])-128);
    }
    if (ord($c[0]) >= 240 && ord($c[0]) <= 247)
    {
        return (ord($c[0])-240)*262144 + (ord($c[1])-128)*4096 + (ord($c[2])-128)*64 + (ord($c[3])-128);
    }
    if (ord($c[0]) >= 248 && ord($c[0]) <= 251)
    {
        return (ord($c[0])-248)*16777216 + (ord($c[1])-128)*262144 + (ord($c[2])-128)*4096 + (ord($c[3])-128)*64 + (ord($c[4])-128);
    }
    if (ord($c[0]) >= 252 && ord($c[0]) <= 253)
    {
        return (ord($c[0])-252)*1073741824 + (ord($c[1])-128)*16777216 + (ord($c[2])-128)*262144 + (ord($c[3])-128)*4096 + (ord($c[4])-128)*64 + (ord($c[5])-128);
    }
    if (ord($c[0]) >= 254 && ord($c[0]) <= 255)
    {
        return 0;
    }
    return 0;
}
function aiomatic_dictZip($x, $y)
{
    $result = array();
    $cnt = 0;
    foreach($x as $i)
    {
        if(isset($i[1]) && isset($i[0]))
        {
            $result[$i[0] . ',' . $i[1]] = $cnt;
            $cnt++;
        }
    }
    return $result;
}
function aiomatic_get_pairs($word) 
{
    $pairs = array();
    $prev_char = $word[0];
    for ($i = 1; $i < count($word); $i++) 
    {
        $char = $word[$i];
        $pairs[] = array($prev_char, $char);
        $prev_char = $char;
    }
    return $pairs;
}
function aiomatic_split($str, $len = 1) 
{
    $arr		= [];
    if(function_exists('mb_strlen'))
    {
        $length 	= mb_strlen($str, 'UTF-8');
    }
    else
    {
        $length 	= strlen($str);
    }

    for ($i = 0; $i < $length; $i += $len) 
    {
        if(function_exists('mb_substr'))
        {
            $arr[] = mb_substr($str, $i, $len, 'UTF-8');
        }
        else
        {
            $arr[] = substr($str, $i, $len);
        }
    }
    return $arr;

}
function aiomatic_bpe($token, $bpe_ranks, &$cache)
{
    if(array_key_exists($token, $cache))
    {
        return $cache[$token];
    }
    $word = aiomatic_split($token);
    $init_len = count($word);
    $pairs = aiomatic_get_pairs($word);
    if(!$pairs)
    {
        return $token;
    }
    while (true) 
    {
        $minPairs = array();
        
        foreach($pairs as $pair)
        {
            if(array_key_exists($pair[0] . ','. $pair[1], $bpe_ranks))
            {
                $rank = $bpe_ranks[$pair[0] . ','. $pair[1]];
                $minPairs[$rank] = $pair;
            }
            else
            { 
                $minPairs[10e10] = $pair;
            }
        }
        ksort($minPairs);
        if(!function_exists('array_key_first'))
        {
            function array_key_first(array $array) { foreach ($array as $key => $value) { return $key; } }
        }
        $min_key = array_key_first($minPairs);
        foreach($minPairs as $mpi => $mp)
        {
            if($mpi < $min_key)
            {
                $min_key = $mpi;
            }
        }
        $bigram = $minPairs[$min_key];
        if(!array_key_exists($bigram[0] . ',' . $bigram[1], $bpe_ranks))
        {
            break;
        }
        $first = $bigram[0];
        $second = $bigram[1];
        $new_word = array();
        $i = 0;
        while ($i < count($word)) 
        {
            $j = aiomatic_indexOf($word, $first, $i);
            if ($j === -1) 
            {
                $new_word = array_merge($new_word, array_slice($word, $i, null, true));
                break;
            }
            if($i > $j)
            {
                $slicer = array();
            }
            elseif($j == 0)
            {
                $slicer = array();
            }
            else
            {
                $slicer = array_slice($word, $i, $j - $i, true);
            }
            $new_word = array_merge($new_word, $slicer);
            if(count($new_word) > $init_len)
            {
                break;
            }
            $i = $j;
            if ($word[$i] === $first && $i < count($word) - 1 && $word[$i + 1] === $second) 
            {
                array_push($new_word, $first . $second);
                $i = $i + 2;
            }
            else
            {
                array_push($new_word, $word[$i]);
                $i = $i + 1;
            }
        }
        if($word == $new_word)
        {
            break;
        }
        $word = $new_word;
        if (count($word) === 1) 
        {
            break;
        }
        else
        {
            $pairs = aiomatic_get_pairs($word);
        }
    }
    $word = implode(' ', $word);
    $cache[$token] = $word;
    return $word;
}
function aiomatic_get_web_page($url)
{
    if(empty($url))
    {
        return false;
    }
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    $content = false;
    if (!isset($aiomatic_Main_Settings['proxy_url']) || $aiomatic_Main_Settings['proxy_url'] == '') 
    {
        $args = array(
        'timeout'     => 180,
        'redirection' => 10,
        'user-agent'  => 'Mozilla/5.0 (Windows NT x.y; Win64; x64; rv:10.0) Gecko/20100101 Firefox/10.0',
        'blocking'    => true,
        'headers'     => array(),
        'cookies'     => array(),
        'body'        => null,
        'compress'    => false,
        'decompress'  => true,
        'sslverify'   => false,
        'stream'      => false,
        'filename'    => null
        );
        $ret_data            = wp_remote_get(html_entity_decode($url), $args);  
        $response_code       = wp_remote_retrieve_response_code( $ret_data );      
        if ( 200 != $response_code ) {
        } else {
            $content = wp_remote_retrieve_body( $ret_data );
        }
    }
    if($content === false)
    {
        if(function_exists('curl_version') && filter_var($url, FILTER_VALIDATE_URL))
        {
            $user_agent = 'Mozilla/5.0 (Windows NT x.y; Win64; x64; rv:10.0) Gecko/20100101 Firefox/10.0';
            $options    = array(
                CURLOPT_CUSTOMREQUEST => "GET",
                CURLOPT_POST => false,
                CURLOPT_USERAGENT => $user_agent,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_HEADER => false,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_ENCODING => "",
                CURLOPT_AUTOREFERER => true,
                CURLOPT_CONNECTTIMEOUT => 10,
                CURLOPT_TIMEOUT => 300,
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_SSL_VERIFYHOST => 0,
                CURLOPT_SSL_VERIFYPEER => 0
            );
            $ch         = curl_init($url);
            if ($ch === FALSE) {
                return FALSE;
            }
            if (isset($aiomatic_Main_Settings['proxy_url']) && $aiomatic_Main_Settings['proxy_url'] != '') {
                $prx = explode(',', $aiomatic_Main_Settings['proxy_url']);
                $randomness = array_rand($prx);
                $options[CURLOPT_PROXY] = trim($prx[$randomness]);
                if (isset($aiomatic_Main_Settings['proxy_auth']) && $aiomatic_Main_Settings['proxy_auth'] != '') 
                {
                    $prx_auth = explode(',', $aiomatic_Main_Settings['proxy_auth']);
                    if(isset($prx_auth[$randomness]) && trim($prx_auth[$randomness]) != '')
                    {
                        $options[CURLOPT_PROXYUSERPWD] = trim($prx_auth[$randomness]);
                    }
                }
            }
            curl_setopt_array($ch, $options);
            $content = curl_exec($ch);
            curl_close($ch);
        }
        else
        {
            $allowUrlFopen = preg_match('/1|yes|on|true/i', ini_get('allow_url_fopen'));
            if ($allowUrlFopen) {
                global $wp_filesystem;
                if ( ! is_a( $wp_filesystem, 'WP_Filesystem_Base') ){
                    include_once(ABSPATH . 'wp-admin/includes/file.php');$creds = request_filesystem_credentials( site_url() );
                    wp_filesystem($creds);
                }
                return $wp_filesystem->get_contents($url);
            }
        }
    }
    return $content;
}
function aiomatic_png_to_jpg_compress($source, $destination, $quality)
{
    $image = imagecreatefrompng($source);
    if($image === false)
    {
        return false;
    }
    $bg = imagecreatetruecolor(imagesx($image), imagesy($image));
    if($bg === false)
    {
        return false;
    }
    $fill = imagefill($bg, 0, 0, imagecolorallocate($bg, 255, 255, 255));
    if($fill === false)
    {
        return false;
    }
    $blend = imagealphablending($bg, TRUE);
    if($blend === false)
    {
        return false;
    }
    $cp = imagecopy($bg, $image, 0, 0, 0, 0, imagesx($image), imagesy($image));
    if($cp === false)
    {
        return false;
    }
    imagedestroy($image);
    $quality = 75;
    $success = imagejpeg($bg, $destination, $quality);
    if($success === false)
    {
        return false;
    }
    imagedestroy($bg);
    return $destination;
}
function aiomatic_string_to_string_compress($image_string) 
{
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if(isset($aiomatic_Main_Settings['disable_compress']) && $aiomatic_Main_Settings['disable_compress'] == 'on')
    {
        return false;
    }
    if(!function_exists('imagecreatefromstring'))
    {
        return false;
    }
    $image = imagecreatefromstring($image_string);
    if ($image === false) {
        return false;
    }
    $bg = imagecreatetruecolor(imagesx($image), imagesy($image));
    if ($bg === false) {
        imagedestroy($image);
        return false;
    }
    $white_background = imagecolorallocate($bg, 255, 255, 255);
    if ($white_background === false) {
        imagedestroy($image);
        return false;
    }
    $fr = imagefilledrectangle($bg, 0, 0, imagesx($image), imagesy($image), $white_background);
    if ($fr === false) {
        imagedestroy($image);
        return false;
    }
    $cr = imagecopy($bg, $image, 0, 0, 0, 0, imagesx($image), imagesy($image));
    if ($cr === false) {
        imagedestroy($image);
        return false;
    }
    ob_start();
    if(isset($aiomatic_Main_Settings['compress_quality']) && $aiomatic_Main_Settings['compress_quality'] != '')
    {
        $quality = intval($aiomatic_Main_Settings['compress_quality']);
    }
    else
    {
        $quality = 75;
    }
    $jp = imagejpeg($bg, NULL, $quality);
    if ($jp === false) {
        imagedestroy($image);
        return false;
    }
    $jpg_image_string = ob_get_contents();
    ob_end_clean();
    imagedestroy($image);
    imagedestroy($bg);
    return $jpg_image_string;
}
function aiomatic_compress_image($source, $destination, $quality) 
{
    $info = getimagesize($source);
    if($info === false)
    {
        return false;
    }
    $image = false;
    if ($info['mime'] == 'image/jpeg') 
    {
        $image = imagecreatefromjpeg($source);
    }
    elseif ($info['mime'] == 'image/gif') 
    {
        $image = imagecreatefromgif($source);
    }
    elseif ($info['mime'] == 'image/png')
    { 
        $image = imagecreatefrompng($source);
    }
    if($image === false)
    {
        return false;
    }
    $success = imagejpeg($image, $destination, $quality);
    if($success === false)
    {
        return false;
    }
    return $destination;
}
function aiomatic_get_web_page_api($url, $post_args = array())
{
    if(count($post_args) == 0)
    {
        $post_args = null;
    }
    $content = false;
    $args = array(
    'method'      => 'POST',
    'timeout'     => 999,
    'redirection' => 10,
    'user-agent'  => 'Mozilla/5.0 (Windows NT x.y; Win64; x64; rv:10.0) Gecko/20100101 Firefox/10.0',
    'blocking'    => true,
    'headers'     => array(),
    'cookies'     => array(),
    'body'        => $post_args,
    'compress'    => false,
    'decompress'  => true,
    'sslverify'   => false,
    'stream'      => false,
    'filename'    => null
    );
    $ret_data            = wp_remote_post($url, $args);  
    $response_code       = wp_remote_retrieve_response_code( $ret_data );     
    if ( 200 != $response_code ) {
    } else {
        $content = wp_remote_retrieve_body( $ret_data );
    }
    if($content === false)
    {
        aiomatic_log_to_file('API response code is: ' . $response_code . ' - ' . $url . ' - ' . print_r($post_args, true));
    }
    return $content;
}

function aiomatic_get_web_page_post($url, $post_args = array(), $headers = array())
{
    if(is_array($post_args) && count($post_args) == 0)
    {
        $post_args = null;
    }
    $content = false;
    $args = array(
        'method'      => 'POST',
        'timeout'     => 999,
        'redirection' => 10,
        'user-agent'  => 'Mozilla/5.0 (Windows NT x.y; Win64; x64; rv:10.0) Gecko/20100101 Firefox/10.0',
        'blocking'    => true,
        'headers'     => $headers,
        'cookies'     => array(),
        'body'        => $post_args,
        'compress'    => false,
        'decompress'  => true,
        'sslverify'   => false,
        'stream'      => false,
        'filename'    => null
    );
    $ret_data            = wp_remote_post($url, $args);  
    $response_code       = wp_remote_retrieve_response_code( $ret_data );     
    if ( 200 != $response_code ) {
    } else {
        $content = wp_remote_retrieve_body( $ret_data );
    }
    if($content === false)
    {
        aiomatic_log_to_file('POST response code is: ' . $response_code . ' - ' . $url . ' - ' . print_r($post_args, true));
    }
    return $content;
}
function aiomatic_utf8_encode($str)
{
    $str .= $str;
    $len = \strlen($str);
    for ($i = $len >> 1, $j = 0; $i < $len; ++$i, ++$j) {
        switch (true) {
            case $str[$i] < "\x80": $str[$j] = $str[$i]; break;
            case $str[$i] < "\xC0": $str[$j] = "\xC2"; $str[++$j] = $str[$i]; break;
            default: $str[$j] = "\xC3"; $str[++$j] = \chr(\ord($str[$i]) - 64); break;
        }
    }
    return substr($str, 0, $j);
}

function aiomatic_indexOf($arrax, $searchElement, $fromIndex)
{
    $index = 0;
    foreach($arrax as $index => $value)
    {
        if($index < $fromIndex)
        {
            $index++;
            continue;
        }
        if($value == $searchElement)
        {
            return $index;
        }
        $index++;
    }
    return -1;
}
function aiomatic_add_to_url($attr, $value, $origURL = '')
{
    if(empty($origURL))
    {
        $origURL = $_SERVER['REQUEST_URI'];
    }
    $url = parse_url($origURL);
    parse_str($url['query'], $q);
    $params = [$attr => $value];
    foreach ( $params as $k => $v ) $q[$k] = $v;
    $new_url = $url['path'] . '?' . http_build_query($q);
    return $new_url;
}
function aiomatic_compute_available_tokens($model, $max_tokens, &$prompt, &$query_token_count)
{
    if(aiomatic_is_chatgpt_turbo_model($model))
    {
        $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
        $available_tokens = $max_tokens;
        if(isset($aiomatic_Main_Settings['gpt4_context_limit']) && $aiomatic_Main_Settings['gpt4_context_limit'] != '')
        {
            if($query_token_count > intval($aiomatic_Main_Settings['gpt4_context_limit']))
            {
                $prompt = aiomatic_strip_to_token_count($prompt, $aiomatic_Main_Settings['gpt4_context_limit'], true);
                $query_token_count = count(aiomatic_encode($prompt));
            }
        }
        elseif($query_token_count > aiomatic_get_max_input_tokens($model))
        {
            $prompt = aiomatic_strip_to_token_count($prompt, aiomatic_get_max_input_tokens($model), true);
            $query_token_count = count(aiomatic_encode($prompt));
        }
    }
    elseif(aiomatic_is_chatgpt35_16k_context_model($model))
    {
        $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
        $available_tokens = $max_tokens;
        if(isset($aiomatic_Main_Settings['gpt35_context_limit']) && $aiomatic_Main_Settings['gpt35_context_limit'] != '')
        {
            if($query_token_count > intval($aiomatic_Main_Settings['gpt35_context_limit']))
            {
                $prompt = aiomatic_strip_to_token_count($prompt, $aiomatic_Main_Settings['gpt35_context_limit'], true);
                $query_token_count = count(aiomatic_encode($prompt));
            }
        }
        elseif($query_token_count > aiomatic_get_max_input_tokens($model))
        {
            $prompt = aiomatic_strip_to_token_count($prompt, aiomatic_get_max_input_tokens($model), true);
            $query_token_count = count(aiomatic_encode($prompt));
        }
    }
    elseif(aiomatic_is_claude_model($model))
    {
        $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
        $available_tokens = $max_tokens;
        if(aiomatic_is_claude_model_200k($model))
        {
            if(isset($aiomatic_Main_Settings['claude_context_limit_200k']) && $aiomatic_Main_Settings['claude_context_limit_200k'] != '')
            {
                if($query_token_count > intval($aiomatic_Main_Settings['claude_context_limit_200k']))
                {
                    $prompt = aiomatic_strip_to_token_count($prompt, $aiomatic_Main_Settings['claude_context_limit_200k'], true);
                    $query_token_count = count(aiomatic_encode($prompt));
                }
            }
            elseif($query_token_count > aiomatic_get_max_input_tokens($model))
            {
                $prompt = aiomatic_strip_to_token_count($prompt, aiomatic_get_max_input_tokens($model), true);
                $query_token_count = count(aiomatic_encode($prompt));
            }
        }
        else
        {
            if(isset($aiomatic_Main_Settings['claude_context_limit']) && $aiomatic_Main_Settings['claude_context_limit'] != '')
            {
                if($query_token_count > intval($aiomatic_Main_Settings['claude_context_limit']))
                {
                    $prompt = aiomatic_strip_to_token_count($prompt, $aiomatic_Main_Settings['claude_context_limit'], true);
                    $query_token_count = count(aiomatic_encode($prompt));
                }
            }
            elseif($query_token_count > aiomatic_get_max_input_tokens($model))
            {
                $prompt = aiomatic_strip_to_token_count($prompt, aiomatic_get_max_input_tokens($model), true);
                $query_token_count = count(aiomatic_encode($prompt));
            }
        }
    }
    elseif(aiomatic_is_perplexity_model($model))
    {
        $available_tokens = $max_tokens - $query_token_count;
        if($available_tokens < 10)
        {
            $prompt = aiomatic_strip_to_token_count($prompt, aiomatic_get_max_input_tokens($model), true);
            $query_token_count = count(aiomatic_encode($prompt));
            $available_tokens = $max_tokens - $query_token_count;
        }
    }
    else
    {
        if($query_token_count > aiomatic_get_max_input_tokens($model))
        {
            $prompt = aiomatic_strip_to_token_count($prompt, aiomatic_get_max_input_tokens($model), true);
            $query_token_count = count(aiomatic_encode($prompt));
        }
        $available_tokens = $max_tokens;
    }
    return $available_tokens;
}
function aiomatic_assign_featured_image($attach_id, $post_id)
{
    if ($attach_id === 0 || !is_numeric($attach_id)) {
        return false;
    }
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    require_once(ABSPATH . 'wp-admin/includes/media.php');
    $res2 = set_post_thumbnail($post_id, $attach_id);
    if ($res2 === FALSE) {
        return false;
    }
    return get_the_post_thumbnail_url($attach_id);
}
function aiomatic_strip_to_token_count($prompt, $max_token_count, $keepend = true)
{
    $tokens = aiomatic_encode($prompt);
    if(count($tokens) > intval($max_token_count))
    {
        if($keepend == true)
        {
            $my_slice = array_slice($tokens, -$max_token_count, $max_token_count, true);
            return aiomatic_decode($my_slice);
        }
        else
        {
            $my_slice = array_slice($tokens, 0, $max_token_count, true);
            return aiomatic_decode($my_slice);
        }
    }
    return $prompt;
}
function aiomatic_is_vision_model($model)
{
    if(in_array($model, AIOMATIC_MODELS_VISION))
    {
        return true;
    }
    return false;
}
function aiomatic_is_claude_model($model)
{
    if(in_array($model, AIOMATIC_CLAUDE_MODELS))
    {
        return true;
    }
    return false;
}
function aiomatic_get_model_provider($model)
{
    if(aiomatic_is_google_model($model))
    {
        return ' (Google)';
    }
    elseif(aiomatic_is_claude_model($model))
    {
        return ' (Anthropic)';
    }
    elseif(aiomatic_is_openrouter_model($model))
    {
        return ' (OpenRouter)';
    }
    elseif(aiomatic_is_perplexity_model($model))
    {
        return ' (PerplexityAI)';
    }
    return '';
}
function aiomatic_is_openrouter_model($model)
{
    $openrouter = false;
    static $router_models = array();
    if(empty($router_models))
    {
        try
        {
            $openrouter = aiomatic_get_openrouter_models();
            if($openrouter !== false)
            {
                foreach($openrouter['source_list'] as $smodel)
                {
                    $router_models[] = $smodel['model'];
                }
            }
        }
        catch(Exception $e)
        {
            aiomatic_log_to_file('Failed to list OpenRouter models: ' . $e->getMessage());
        }
    }
    if(in_array($model, $router_models))
    {
        return true;
    }
    return false;
}
function aiomatic_is_perplexity_model($model)
{
    $modellist = AIOMATIC_PERPLEXITY_MODELS;
    if(in_array($model, $modellist))
    {
        return true;
    }
    return false;
}
function aiomatic_is_google_model($model)
{
    if(in_array($model, AIOMATIC_GOOGLE_MODELS))
    {
        return true;
    }
    return false;
}
function aiomatic_is_claude_model_200k($model)
{
    if(in_array($model, AIOMATIC_CLAUDE_MODELS_200K))
    {
        return true;
    }
    return false;
}
function aiomatic_is_claude_3_model($model)
{
    if(in_array($model, AIOMATIC_CLAUDE_CHAT))
    {
        return true;
    }
    return false;
}
function aiomatic_is_new_token_window_model($model)
{
    if(aiomatic_is_chatgpt_turbo_model($model) || aiomatic_is_chatgpt35_16k_context_model($model))
    {
        return true;
    }
    return false;
}
function aiomatic_is_chatgpt_turbo_model($model)
{
    if($model == 'gpt-4-1106-preview')
    {
        return true;
    }
    elseif($model == 'gpt-4-vision-preview')
    {
        return true;
    }
    elseif($model == 'gpt-4-0125-preview')
    {
        return true;
    }
    elseif(stristr($model, 'gpt-4-turbo') !== false)
    {
        return true;
    }
    return false;
}
function aiomatic_is_chatgpt35_16k_context_model($model)
{
    if($model == 'gpt-3.5-turbo-1106')
    {
        return true;
    }
    elseif($model == 'gpt-3.5-turbo-0125')
    {
        return true;
    }
    return false;
}
function aiomatic_is_chatgpt_model($model)
{
    if(stristr($model, 'gpt-3.5-turbo-instruct') !== false)
    {
        return false;
    }
    elseif(stristr($model, 'gpt-4-1106-preview') !== false)
    {
        return false;
    }
    elseif(stristr($model, 'gpt-4-0125-preview') !== false)
    {
        return false;
    }
    elseif(stristr($model, 'gpt-4-turbo') !== false)
    {
        return false;
    }
    elseif(stristr($model, 'gpt-4-vision-preview') !== false)
    {
        return false;
    }
    elseif ( preg_match('/^((?:gpt)-(?:[\d.]+)(?:-[a-zA-Z0-9]+)?(?:-[\d]+)?)/', $model, $matches ) || preg_match('/^ft:((?:gpt)-(?:[\d.]+)(?:-[a-zA-Z0-9]+)?(?:-[\d]+)?):.*/', $model, $matches ) )
    {
        return true;
    }
    return false;
}
function aiomatic_get_all_models_claude($reverse = false)
{
    $all_models = AIOMATIC_CLAUDE_MODELS;
    if($reverse)
    {
        $all_models = array_reverse($all_models);
    }
    return $all_models;
}
function aiomatic_get_all_models($reverse = false)
{
    $all_models = array();
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if(!function_exists('is_plugin_active'))
    {
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    }
    if(isset($aiomatic_Main_Settings['app_id_openrouter']) && !empty(trim($aiomatic_Main_Settings['app_id_openrouter'])))
    {
        try
        {
            $openrouter = aiomatic_get_openrouter_models();
            if($openrouter !== false)
            {
                foreach($openrouter['source_list'] as $smodel)
                {
                    $all_models[] = $smodel['model'];
                }
            }
        }
        catch(Exception $e)
        {
            aiomatic_log_to_file('Failed to list OpenRouter models: ' . $e->getMessage());
        }
    }
    if(isset($aiomatic_Main_Settings['app_id_perplexity']) && !empty(trim($aiomatic_Main_Settings['app_id_perplexity'])))
    {
        $all_models = array_merge(AIOMATIC_PERPLEXITY_MODELS, $all_models);
    }
    if(isset($aiomatic_Main_Settings['app_id_claude']) && !empty(trim($aiomatic_Main_Settings['app_id_claude'])))
    {
        $all_models = array_merge(AIOMATIC_CLAUDE_MODELS, $all_models);
    }
    if(isset($aiomatic_Main_Settings['app_id_google']) && !empty(trim($aiomatic_Main_Settings['app_id_google'])))
    {
        $all_models = array_merge(AIOMATIC_GOOGLE_MODELS, $all_models);
    }
    $all_custom_models = get_option('aiomatic_custom_models', array());
    if($reverse == true)
    {
        $all_models = array_merge($all_custom_models, $all_models);
        $all_models = array_merge(AIOMATIC_MODELS, $all_models);
        $all_models = array_merge(AIOMATIC_MODELS_CHAT, $all_models);
    }
    else
    {
        $all_models = array_merge(AIOMATIC_MODELS_CHAT, $all_models);
        $all_models = array_merge(AIOMATIC_MODELS, $all_models);
        $all_models = array_merge($all_custom_models, $all_models);
    }
    return $all_models;
}
function aiomatic_get_all_models_function()
{
    $all_models = AIOMATIC_FUNCTION_CALLING_MODELS;
    return $all_models;
}
function aiomatic_get_all_assistants($reverse = false)
{
    $aiomatic_Main_Settings = get_option('aiomatic_Main_Settings', false);
    if(!isset($aiomatic_Main_Settings['app_id']) || empty(trim($aiomatic_Main_Settings['app_id'])))
    {
        return array();
    }
    $appids = preg_split('/\r\n|\r|\n/', trim($aiomatic_Main_Settings['app_id']));
    $appids = array_filter($appids);
    $token = $appids[array_rand($appids)];
    if(aiomatic_is_aiomaticapi_key($token))
    {
        return false;
    }
    else
    {
        if (aiomatic_check_if_azure_or_others($aiomatic_Main_Settings))
        {
            return false;
        }
    }
    $assistant_list = array();
    $postsPerPage = 50000;
    $paged = 0;
    do
    {
        $postOffset = $paged * $postsPerPage;
        $query = array(
            'post_status' => array(
                'publish'
            ),
            'post_type' => array(
                'aiomatic_assistants'
            ),
            'numberposts' => $postsPerPage,
            'offset'  => $postOffset
        );
        $got_me = get_posts($query);
        $assistant_list = array_merge($assistant_list, $got_me);
        $paged++;
    }while(!empty($got_me));
    if($reverse)
    {
        $assistant_list = array_reverse($assistant_list);
    }
    return $assistant_list;
}
function aiomatic_starts_with($newx_url, $query)
{
    if(substr( $newx_url, 0, strlen($query) ) === $query)
    {
        return true;
    }
    return false;
}
function aiomatic_get_word($key, $r_vars)
{
    if (isset($r_vars[$key])) {
        
        $words  = $r_vars[$key];
        $w_max  = count($words) - 1;
        $w_rand = rand(0, $w_max);
        return aiomatic_replace_words(trim($words[$w_rand]), $r_vars);
    } else {
        return "";
    }
    
}

function aiomatic_replace_words($sentence, $r_vars)
{
    
    if (str_replace('%', '', $sentence) == $sentence)
        return $sentence;
    
    $words = explode(" ", $sentence);
    
    $new_sentence = array();
    for ($w = 0; $w < count($words); $w++) {
        
        $word = trim($words[$w]);
        
        if ($word != '') {
            if (preg_match('/^%([^%\n]*)$/', $word, $m)) {
                $varkey         = trim($m[1]);
                $new_sentence[] = aiomatic_get_word($varkey, $r_vars);
            } else {
                $new_sentence[] = $word;
            }
        }
    }
    return implode(" ", $new_sentence);
}
function aiomatic_get_plugin_url()
{
    return plugins_url('', __FILE__);
}

function aiomatic_get_file_url($url)
{
    return esc_url_raw(aiomatic_get_plugin_url() . '/' . $url);
}
function aiomatic_redirect($url, $statusCode = 301)
{
  if(!function_exists('wp_redirect'))
  {
     include_once( ABSPATH . 'wp-includes/pluggable.php' );
  }
  wp_redirect($url, $statusCode);
  die();
}
function aiomatic_sanitize_date_time( $date_time, $type = 'date', $accepts_string = false ) {
	if ( empty( $date_time ) || ! in_array( $type, array( 'date', 'time' ) ) ) {
		return array();
	}
	$segments = array();
	if (
		true === $accepts_string
		&& ( false !== strpos( $date_time, ' ' ) || false === strpos( $date_time, '-' ) )
	) {
		if ( false !== $timestamp = strtotime( $date_time ) ) {
			return $date_time;
		}
	}
	$parts = array_map( 'absint', explode( 'date' == $type ? '-' : ':', $date_time ) );
	if ( 'date' == $type ) {
		$year = $month = $day = 1;
		if ( count( $parts ) >= 3 ) {
			list( $year, $month, $day ) = $parts;
			$year  = ( $year  >= 1 && $year  <= 9999 ) ? $year  : 1;
			$month = ( $month >= 1 && $month <= 12   ) ? $month : 1;
			$day   = ( $day   >= 1 && $day   <= 31   ) ? $day   : 1;
		}
		$segments = array(
			'year'  => $year,
			'month' => $month,
			'day'   => $day
		);
	} elseif ( 'time' == $type ) {
		$hour = $minute = $second = 0;
		switch( count( $parts ) ) {
			case 3 :
				list( $hour, $minute, $second ) = $parts;
				$hour   = ( $hour   >= 0 && $hour   <= 23 ) ? $hour   : 0;
				$minute = ( $minute >= 0 && $minute <= 60 ) ? $minute : 0;
				$second = ( $second >= 0 && $second <= 60 ) ? $second : 0;
				break;
			case 2 :
				list( $hour, $minute ) = $parts;
				$hour   = ( $hour   >= 0 && $hour   <= 23 ) ? $hour   : 0;
				$minute = ( $minute >= 0 && $minute <= 60 ) ? $minute : 0;
				break;
			default : break;
		}
		$segments = array(
			'hour'   => $hour,
			'minute' => $minute,
			'second' => $second
		);
	}

	return apply_filters( 'display_posts_shortcode_sanitized_segments', $segments, $date_time, $type );
}
?>